@@ -27,7 +27,6 @@
 #include "ifilesystem.h"
 #include "ifiletypes.h"
 #include "ifilter.h"
-#include "ifiltermenu.h"
 #include "ifonts.h"
 #include "igame.h"
 #include "igl.h"
@@ -84,7 +83,6 @@
 #include "iselectiontest.h"
 #include "ishaderclipboard.h"
 #include "ishaderexpression.h"
-#include "ishaders.h"
 #include "isound.h"
 #include "ispacepartition.h"
 #include "itexdef.h"
@@ -92,13 +90,12 @@
 #include "itraceable.h"
 #include "itransformable.h"
 #include "itransformnode.h"
-#include "iuimanager.h"
 #include "iundo.h"
 #include "ivolumetest.h"
 #include "mapfile.h"
 #include "modelskin.h"
 #include "ModResource.h"
-#include "ShaderLayer.h"
+#include "ishaderlayer.h"
 #include "StringSerialisable.h"
 #include "Texture.h"
 #include "VolumeIntersectionValue.h"
