@@ -71,11 +71,11 @@ public:
 
     void setRotation(const Quaternion& value, const Vector3& worldPivot, const Matrix4& localToWorld) override
     {
-        // greebo: When rotating around a pivot, the operation can be split into a rotation 
+        // greebo: When rotating around a pivot, the operation can be split into a rotation
         // and a translation part. Calculate the translation part and apply it.
 
         // Translate the world pivot into local coordinates (we only care about the translation part)
-        Vector3 localPivot = worldPivot - localToWorld.t().getVector3();
+        Vector3 localPivot = worldPivot - localToWorld.tCol().getVector3();
 
         Matrix4 rotation = Matrix4::getRotationQuantised(value);
 
@@ -195,12 +195,12 @@ private:
 	static Matrix4 getMatrixForComponents(const Vector3& translation, const Quaternion& rotation, const Vector3& scale)
 	{
 		Matrix4 result(Matrix4::getRotationQuantised(rotation));
-		result.x().getVector3() *= scale.x();
-		result.y().getVector3() *= scale.y();
-		result.z().getVector3() *= scale.z();
+		result.xCol().getVector3() *= scale.x();
+		result.yCol().getVector3() *= scale.y();
+		result.zCol().getVector3() *= scale.z();
 		result.tx() = translation.x();
 		result.ty() = translation.y();
 		result.tz() = translation.z();
 		return result;
-	} 
+	}
 };
