@@ -5,9 +5,9 @@
 
 double Plane3::distanceToOrientedExtents(const Vector3& extents, const Matrix4& orientation) const
 {
-	return fabs(extents[0] * normal().dot(orientation.x().getVector3())) + 
-		   fabs(extents[1] * normal().dot(orientation.y().getVector3())) + 
-		   fabs(extents[2] * normal().dot(orientation.z().getVector3()));
+	return fabs(extents[0] * normal().dot(orientation.xCol().getVector3())) +
+		   fabs(extents[1] * normal().dot(orientation.yCol().getVector3())) +
+		   fabs(extents[2] * normal().dot(orientation.zCol().getVector3()));
 }
 
 bool Plane3::containsAABB(const AABB& aabb, const Matrix4& orientation) const
