@@ -89,6 +89,11 @@ bool Node::checkStateFlag(unsigned int state) const
     return (_state & state) != 0;
 }
 
+bool Node::supportsStateFlag(unsigned int state) const
+{
+    return true;
+}
+
 bool Node::visible() const
 {
 	// Only instantiated nodes can be considered visible
@@ -440,11 +445,6 @@ void Node::transformChangedLocal()
 	_transformMutex = false;
 	_boundsChanged = true;
 	_childBoundsChanged = true;
-
-	if (_transformChangedCallback)
-	{
-		_transformChangedCallback();
-	}
 }
 
 void Node::transformChanged()
@@ -462,10 +462,6 @@ void Node::transformChanged()
 	boundsChanged();
 }
 
-void Node::setTransformChangedCallback(const Callback& callback) {
-	_transformChangedCallback = callback;
-}
-
 RenderSystemPtr Node::getRenderSystem() const
 {
 	return _renderSystem.lock();
