@@ -9,12 +9,12 @@
 #include "i18n.h"
 #include "ifavourites.h"
 #include "imainframe.h"
-#include "iuimanager.h"
 #include "gamelib.h"
 
 #include <wx/button.h>
 #include <wx/panel.h>
 #include <wx/splitter.h>
+#include "wxutil/Bitmap.h"
 
 #include "string/string.h"
 #include "eclass.h"
@@ -27,7 +27,9 @@ namespace wxutil
 namespace
 {
     const char* const ECLASS_CHOOSER_TITLE = N_("Create entity");
-    const std::string RKEY_SPLIT_POS = "user/ui/entityClassChooser/splitPos";
+    const char* const RKEY_SPLIT_POS = "user/ui/entityClassChooser/splitPos";
+    const char* const RKEY_WINDOW_STATE = "user/ui/entityClassChooser/window";
+    const char* const RKEY_LAST_SELECTED_ECLASS = "user/ui/entityClassChooser/lastSelectedEclass";
 
     const char* const FOLDER_ICON = "folder16.png";
     const char* const ENTITY_ICON = "cmenu_add_entity.png";
@@ -68,8 +70,8 @@ public:
       _columns(columns),
       _folderKey(game::current::getValue<std::string>(FOLDER_KEY_PATH))
     {
-        _folderIcon.CopyFromBitmap(wxArtProvider::GetBitmap(GlobalUIManager().ArtIdPrefix() + FOLDER_ICON));
-        _entityIcon.CopyFromBitmap(wxArtProvider::GetBitmap(GlobalUIManager().ArtIdPrefix() + ENTITY_ICON));
+        _folderIcon.CopyFromBitmap(wxutil::GetLocalBitmap(FOLDER_ICON));
+        _entityIcon.CopyFromBitmap(wxutil::GetLocalBitmap(ENTITY_ICON));
 
         // Get the list of favourite eclasses
         _favourites = GlobalFavouritesManager().getFavourites(decl::Type::EntityDef);
@@ -89,7 +91,7 @@ public:
         // of the DISPLAY_FOLDER_KEY.
         addPath(
             eclass->getModName() + folderPath + "/" + eclass->getName(),
-            [&](TreeModel::Row& row, const std::string& path, 
+            [&](TreeModel::Row& row, const std::string& path,
                 const std::string& leafName, bool isFolder)
             {
                 bool isFavourite = !isFolder && _favourites.count(leafName) > 0;
@@ -144,7 +146,7 @@ public:
 };
 
 // Main constructor
-EntityClassChooser::EntityClassChooser() : 
+EntityClassChooser::EntityClassChooser() :
     DialogBase(_(ECLASS_CHOOSER_TITLE)),
     _treeView(nullptr),
     _selectedName("")
@@ -180,8 +182,6 @@ EntityClassChooser::EntityClassChooser() :
     makeLabelBold(this, "EntityClassChooserDefFileNameLabel");
     makeLabelBold(this, "EntityClassChooserUsageLabel");
 
-    FitToScreen(0.7f, 0.8f);
-
     wxSplitterWindow* splitter = findNamedObject<wxSplitterWindow>(this, "EntityClassChooserSplitter");
 
     // Disallow unsplitting
@@ -189,6 +189,8 @@ EntityClassChooser::EntityClassChooser() :
     splitter->SetSashPosition(static_cast<int>(GetSize().GetWidth() * 0.2f));
 
     // Persist layout to registry
+    _windowPosition.initialise(this, RKEY_WINDOW_STATE, 0.7f, 0.8f);
+
     _panedPosition.connect(splitter);
     _panedPosition.loadFromPath(RKEY_SPLIT_POS);
 
@@ -200,58 +202,39 @@ EntityClassChooser::EntityClassChooser() :
         wxSize(GetSize().GetWidth() * 0.4f, GetSize().GetHeight() * 0.2f));
 }
 
-// Display the singleton instance
-std::string EntityClassChooser::chooseEntityClass(const std::string& preselectEclass)
+EntityClassChooser::~EntityClassChooser()
 {
+    _defsReloaded.disconnect();
+}
+
+// Display the singleton instance
+std::string EntityClassChooser::chooseEntityClass(const std::string& eclassToSelect)
+{
+    EntityClassChooser instance;
+
+    // Fall back to the value we saved in the registry if we didn't get any other instructions
+    auto preselectEclass = !eclassToSelect.empty() ? eclassToSelect : 
+        registry::getValue<std::string>(RKEY_LAST_SELECTED_ECLASS);
+
     if (!preselectEclass.empty())
     {
-        Instance().setSelectedEntityClass(preselectEclass);
+        instance.setSelectedEntityClass(preselectEclass);
     }
 
-    if (Instance().ShowModal() == wxID_OK)
+    if (instance.ShowModal() == wxID_OK)
     {
-        return Instance().getSelectedEntityClass();
+        auto selection = instance.getSelectedEntityClass();
+
+        // Remember this selection on OK
+        if (!selection.empty())
+        {
+            registry::setValue(RKEY_LAST_SELECTED_ECLASS, selection);
+        }
+
+        return selection;
     }
-    else
-    {
-        return ""; // Empty selection on cancel
-    }
-}
-
-EntityClassChooser& EntityClassChooser::Instance()
-{
-    EntityClassChooserPtr& instancePtr = InstancePtr();
-
-    if (!instancePtr)
-    {
-        // Not yet instantiated, do it now
-        instancePtr.reset(new EntityClassChooser);
-
-        // Pre-destruction cleanup
-        GlobalMainFrame().signal_MainFrameShuttingDown().connect(
-            sigc::mem_fun(*instancePtr, &EntityClassChooser::onMainFrameShuttingDown)
-        );
-    }
-
-    return *instancePtr;
-}
-
-EntityClassChooserPtr& EntityClassChooser::InstancePtr()
-{
-    static EntityClassChooserPtr _instancePtr;
-    return _instancePtr;
-}
-
-void EntityClassChooser::onMainFrameShuttingDown()
-{
-    rMessage() << "EntityClassChooser shutting down." << std::endl;
-
-    _modelPreview.reset();
-    _defsReloaded.disconnect();
-
-    // Final step at shutdown, release the shared ptr
-    Instance().SendDestroyEvent();
-    InstancePtr().reset();
+    
+    return ""; // Empty selection on cancel
 }
 
 void EntityClassChooser::loadEntityClasses()
@@ -281,6 +264,8 @@ void EntityClassChooser::onDeleteEvent(wxCloseEvent& ev)
 
 int EntityClassChooser::ShowModal()
 {
+    _windowPosition.applyPosition();
+
     _treeViewToolbar->ClearFilter();
 
     // Update the member variables
@@ -292,6 +277,7 @@ int EntityClassChooser::ShowModal()
     int returnCode = DialogBase::ShowModal();
 
     _panedPosition.saveToPath(RKEY_SPLIT_POS);
+    _windowPosition.saveToPath(RKEY_WINDOW_STATE);
 
     return returnCode;
 }
