@@ -1,11 +1,11 @@
 #include "ResourceTreeView.h"
 
 #include "i18n.h"
-#include "iuimanager.h"
+#include "iclipboard.h"
 #include "ifavourites.h"
 #include "../menu/IconTextMenuItem.h"
 #include "TreeViewItemStyle.h"
-#include <wx/artprov.h>
+#include "wxutil/Bitmap.h"
 
 namespace wxutil
 {
@@ -44,7 +44,8 @@ ResourceTreeView::ResourceTreeView(wxWindow* parent, const TreeModel::Ptr& model
     _mode(TreeMode::ShowAll),
     _expandTopLevelItemsAfterPopulation(false),
     _columnToSelectAfterPopulation(nullptr),
-    _declType(decl::Type::None)
+    _declType(decl::Type::None),
+    _declPathColumn(_columns.fullName)
 {
     _treeStore = model;
 
@@ -60,7 +61,7 @@ ResourceTreeView::ResourceTreeView(wxWindow* parent, const TreeModel::Ptr& model
     Bind(EV_TREEMODEL_POPULATION_FINISHED, &ResourceTreeView::_onTreeStorePopulationFinished, this);
     Bind(EV_TREEMODEL_POPULATION_PROGRESS, &ResourceTreeView::_onTreeStorePopulationProgress, this);
 
-    _progressIcon.CopyFromBitmap(wxArtProvider::GetBitmap(GlobalUIManager().ArtIdPrefix() + ICON_LOADING));
+    _progressIcon.CopyFromBitmap(wxutil::GetLocalBitmap(ICON_LOADING));
 }
 
 ResourceTreeView::~ResourceTreeView()
@@ -101,10 +102,8 @@ void ResourceTreeView::SetupTreeModelFilter()
     // Set up the filter
     _treeModelFilter.reset(new TreeModelFilter(_treeStore));
 
-    _treeModelFilter->SetVisibleFunc([this](TreeModel::Row& row)
-    {
-        return IsTreeModelRowVisible(row);
-    });
+    _treeModelFilter->SetVisibleFunc(
+        std::bind(&ResourceTreeView::IsTreeModelRowOrAnyChildVisible, this, std::placeholders::_1));
 
     AssociateModel(_treeModelFilter.get());
 
@@ -127,7 +126,7 @@ void ResourceTreeView::SetupTreeModelFilter()
                 _emptyFavouritesLabel = row.getItem();
 
                 wxIcon icon;
-                icon.CopyFromBitmap(wxArtProvider::GetBitmap(GlobalUIManager().ArtIdPrefix() + ICON_LOADING));
+                icon.CopyFromBitmap(wxutil::GetLocalBitmap(ICON_LOADING));
                 row[_columns.iconAndName] = wxVariant(wxDataViewIconText(_("No favourites added so far"), icon));
                 row[_columns.isFavourite] = true;
                 row[_columns.isFolder] = false;
@@ -195,6 +194,15 @@ void ResourceTreeView::PopulateContextMenu(wxutil::PopupMenu& popupMenu)
         std::bind(&ResourceTreeView::_testRemoveFromFavourites, this),
         [this]() { return _declType != decl::Type::None; }
     );
+
+    popupMenu.addSeparator();
+
+    popupMenu.addItem(
+        new wxutil::StockIconTextMenuItem(_("Copy Resource Path"), wxART_COPY),
+        std::bind(&ResourceTreeView::_onCopyResourcePath, this),
+        std::bind(&ResourceTreeView::_copyResourcePathEnabled, this),
+        std::bind(&ResourceTreeView::_copyResourcePathVisible, this)
+    );
 }
 
 bool ResourceTreeView::SetFilterText(const wxString& filterText)
@@ -231,14 +239,21 @@ void ResourceTreeView::UpdateTreeVisibility()
 {
     if (_treeModelFilter)
     {
-        // Notify the attached views that it should reload
+#if defined(__WXGTK__) && !wxCHECK_VERSION(3, 0, 5)
+        // In wxGTK 3.0.4 Cleared() will just wipe out the treeview
+        // Re-associate the model to refresh the view
+        AssociateModel(nullptr);
+        AssociateModel(_treeModelFilter.get());
+#else
+        // Notify the attached view that it should reload
         _treeModelFilter->Cleared();
+#endif
     }
 }
 
 bool ResourceTreeView::JumpToFirstFilterMatch()
 {
-    if (_filterText.empty()) return false;
+    if (_filterText.empty() || !_treeModelFilter) return false;
 
     auto item = _treeModelFilter->FindNextString(_filterText, _colsToSearch);
 
@@ -514,6 +529,49 @@ void ResourceTreeView::_onSetFavourite(bool isFavourite)
     SetFavouriteRecursively(row, isFavourite);
 }
 
+std::string ResourceTreeView::GetResourcePath(const TreeModel::Row& row)
+{
+    return row[_declPathColumn];
+}
+
+void ResourceTreeView::SetDeclPathColumn(const TreeModel::Column& declPathColumn)
+{
+    _declPathColumn = declPathColumn;
+}
+
+std::string ResourceTreeView::GetResourcePathOfSelection()
+{
+    wxDataViewItem item = GetSelection();
+
+    if (!item.IsOk() || IsDirectorySelected())
+    {
+        return std::string();
+    }
+
+    TreeModel::Row row(item, *GetModel());
+    return GetResourcePath(row);
+}
+
+void ResourceTreeView::_onCopyResourcePath()
+{
+    auto resourcePath = GetResourcePathOfSelection();
+
+    if (!resourcePath.empty())
+    {
+        GlobalClipboard().setString(resourcePath);
+    }
+}
+
+bool ResourceTreeView::_copyResourcePathEnabled()
+{
+    return !GetResourcePathOfSelection().empty();
+}
+
+bool ResourceTreeView::_copyResourcePathVisible()
+{
+    return !IsDirectorySelected() && module::GlobalModuleRegistry().moduleExists(MODULE_CLIPBOARD);
+}
+
 bool ResourceTreeView::IsDirectorySelected()
 {
     wxDataViewItem item = GetSelection();
@@ -536,73 +594,56 @@ bool ResourceTreeView::IsFavouriteSelected()
     return row[_columns.isFavourite].getBool();
 }
 
+bool ResourceTreeView::IsTreeModelRowOrAnyChildVisible(TreeModel::Row& row)
+{
+    // Test the node itself
+    if (IsTreeModelRowVisible(row))
+    {
+        return true; // node is visible
+    }
+
+    // The node itself is invisible, but it might still be visible
+    // if any of the child nodes is visible, dive into it
+    wxDataViewItemArray children;
+    _treeStore->GetChildren(row.getItem(), children);
+
+    for (const wxDataViewItem& child : children)
+    {
+        wxutil::TreeModel::Row childRow(child, *_treeStore);
+
+        // Check the child node (recursively)
+        if (IsTreeModelRowOrAnyChildVisible(childRow))
+        {
+            return true; // found a visible child, this is enough
+        }
+    }
+
+    // Either we don't have any children, or 
+    // all child nodes are invisible, so this node is invisible too
+    return false;
+}
+
 bool ResourceTreeView::IsTreeModelRowVisible(wxutil::TreeModel::Row& row)
 {
+    // Check view mode
     if (!IsTreeModelRowVisibleByViewMode(row))
     {
         return false; // done here
     }
 
-    // Check for a text filter we might need to apply
-    if (_filterText.empty())
-    {
-        return true; // done here
-    }
-
-    return !IsTreeModelRowFilteredRecursively(row);
+    // Check filter, otherwise the node is visible
+    return !IsTreeModelRowFiltered(row);
 }
 
-bool ResourceTreeView::IsTreeModelRowFilteredRecursively(wxutil::TreeModel::Row& row)
+bool ResourceTreeView::IsTreeModelRowFiltered(wxutil::TreeModel::Row& row)
 {
-    if (TreeModel::RowContainsString(row, _filterText, _colsToSearch, true))
-    {
-        return false;
-    }
-
-    // This node might also be visible if a single child node is visible, dive into it
-    wxDataViewItemArray children;
-    _treeStore->GetChildren(row.getItem(), children);
-
-    for (const wxDataViewItem& child : children)
-    {
-        wxutil::TreeModel::Row childRow(child, *_treeStore);
-
-        if (!IsTreeModelRowFilteredRecursively(childRow))
-        {
-            return false; // unfiltered node, break the loop
-        }
-    }
-
-    // Either we don't have any children, or 
-    // all child nodes are filtered, so this node is filtered too
-    return true;
+    return !_filterText.empty() && !TreeModel::RowContainsString(row, _filterText, _colsToSearch, true);
 }
 
 bool ResourceTreeView::IsTreeModelRowVisibleByViewMode(wxutil::TreeModel::Row& row)
 {
-    if (_mode == TreeMode::ShowAll) return true; // everything is visible
-
-    // Favourites mode, check if this item or any descendant is visible
-    if (row[_columns.isFavourite].getBool())
-    {
-        return true;
-    }
-
-    wxDataViewItemArray children;
-    _treeStore->GetChildren(row.getItem(), children);
-
-    // Enter the recursion for each of the children and bail out on the first visible one
-    for (const wxDataViewItem& child : children)
-    {
-        wxutil::TreeModel::Row childRow(child, *_treeStore);
-
-        if (IsTreeModelRowVisibleByViewMode(childRow))
-        {
-            return true;
-        }
-    }
-
-    return false;
+    // In favourites mode, check if this item is marked as favourite
+    return _mode == TreeMode::ShowAll || row[_columns.isFavourite].getBool();
 }
 
 }
