@@ -20,8 +20,6 @@ namespace wxutil
 class DialogBase :
     public wxDialog
 {
-    void _onDelete(wxCloseEvent& ev);
-
 public:
     /// Construct and initialise
     DialogBase(const std::string& title, wxWindow* parent = NULL);
@@ -34,6 +32,8 @@ public:
      */
     void FitToScreen(float xProp, float yProp);
 
+    virtual int ShowModal() override;
+
 protected:
     // Overrideable: return true to prevent the window from being deleted
     virtual bool _onDeleteEvent()
