@@ -5,18 +5,20 @@
 #include "icameraview.h"
 #include "iscenegraphfactory.h"
 #include "irendersystemfactory.h"
-#include "iuimanager.h"
 
 #include "math/AABB.h"
 #include "util/ScopedBoolLock.h"
 #include "registry/registry.h"
 #include "render/CamRenderer.h"
+#include "render/CameraView.h"
 #include "render/SceneRenderWalker.h"
+#include "wxutil/menu/FilterPopupMenu.h"
 
 #include "../GLWidget.h"
 #include <wx/sizer.h>
 #include <wx/menu.h>
 #include <wx/dcclient.h>
+#include "../Bitmap.h"
 
 #include <fmt/format.h>
 #include <functional>
@@ -27,7 +29,6 @@ namespace wxutil
 namespace
 {
     const GLfloat PREVIEW_FOV = 60;
-    const unsigned int MSEC_PER_FRAME = 16;
 
     // Widget names
     const std::string BOTTOM_BOX("bottomBox");
@@ -55,7 +56,6 @@ RenderPreview::RenderPreview(wxWindow* parent, bool enableAnimation) :
     _timer(this),
     _previewWidth(0),
     _previewHeight(0),
-    _filtersMenu(GlobalUIManager().createFilterMenu()),
 	_filterTool(nullptr)
 {
 	Bind(wxEVT_TIMER, &RenderPreview::_onFrame, this);
@@ -99,14 +99,12 @@ void RenderPreview::setupToolbars(bool enableAnimation)
     // Connect filters menu to toolbar
     wxToolBar* filterToolbar = findNamedObject<wxToolBar>(_mainPanel, "RenderPreviewFilterToolbar");
 
-    wxMenu* filterSubmenu = _filtersMenu->getMenuWidget();
-
     wxToolBarToolBase* filterTool = filterToolbar->AddTool(wxID_ANY, _("Filters"),
-                                                           wxArtProvider::GetBitmap(GlobalUIManager().ArtIdPrefix() + "iconFilter16.png"),
+                                                           wxutil::GetLocalBitmap("iconFilter16.png"),
                                                            _("Filters"), wxITEM_DROPDOWN);
 
-	// By setting it as dropdown menu the toolitem will take ownership and delete the menu on destruction
-    filterToolbar->SetDropdownMenu(filterTool->GetId(), filterSubmenu);
+	// By setting a drodown menu the toolitem will take ownership and delete the menu on destruction
+    filterToolbar->SetDropdownMenu(filterTool->GetId(), new wxutil::FilterPopupMenu());
 
     filterToolbar->Realize();
 
@@ -117,7 +115,7 @@ void RenderPreview::setupToolbars(bool enableAnimation)
 
     wxToolBar* renderToolbar = findNamedObject<wxToolBar>(_mainPanel, "RenderPreviewRenderModeToolbar");
 
-    renderToolbar->Bind(wxEVT_TOOL, &RenderPreview::onRenderModeChanged, this, 
+    renderToolbar->Bind(wxEVT_TOOL, &RenderPreview::onRenderModeChanged, this,
         getToolBarToolByLabel(renderToolbar, "texturedModeButton")->GetId());
     renderToolbar->Bind(wxEVT_TOOL, &RenderPreview::onRenderModeChanged, this,
         getToolBarToolByLabel(renderToolbar, "lightingModeButton")->GetId());
@@ -136,16 +134,16 @@ void RenderPreview::connectToolbarSignals()
 {
 	wxToolBar* toolbar = findNamedObject<wxToolBar>(_mainPanel, "RenderPreviewAnimToolbar");
 
-	toolbar->Connect(getToolBarToolByLabel(toolbar, "startTimeButton")->GetId(), 
+	toolbar->Connect(getToolBarToolByLabel(toolbar, "startTimeButton")->GetId(),
 		wxEVT_TOOL, wxCommandEventHandler(RenderPreview::onStartPlaybackClick), NULL, this);
-	toolbar->Connect(getToolBarToolByLabel(toolbar, "pauseTimeButton")->GetId(), 
+	toolbar->Connect(getToolBarToolByLabel(toolbar, "pauseTimeButton")->GetId(),
 		wxEVT_TOOL, wxCommandEventHandler(RenderPreview::onPausePlaybackClick), NULL, this);
-	toolbar->Connect(getToolBarToolByLabel(toolbar, "stopTimeButton")->GetId(), 
+	toolbar->Connect(getToolBarToolByLabel(toolbar, "stopTimeButton")->GetId(),
 		wxEVT_TOOL, wxCommandEventHandler(RenderPreview::onStopPlaybackClick), NULL, this);
 
-	toolbar->Connect(getToolBarToolByLabel(toolbar, "prevButton")->GetId(), 
+	toolbar->Connect(getToolBarToolByLabel(toolbar, "prevButton")->GetId(),
 		wxEVT_TOOL, wxCommandEventHandler(RenderPreview::onStepBackClick), NULL, this);
-	toolbar->Connect(getToolBarToolByLabel(toolbar, "nextButton")->GetId(), 
+	toolbar->Connect(getToolBarToolByLabel(toolbar, "nextButton")->GetId(),
 		wxEVT_TOOL, wxCommandEventHandler(RenderPreview::onStepForwardClick), NULL, this);
 }
 
@@ -220,6 +218,9 @@ void RenderPreview::initialisePreview()
         setLightingModeEnabled(false);
     }
 
+    // In case it didn't happen until now, make sure the rendersystem is realised
+    _renderSystem->realise();
+
     updateModelViewMatrix();
 }
 
@@ -254,6 +255,20 @@ void RenderPreview::setLightingModeEnabled(bool enabled)
         _renderSystem->setShaderProgram(RenderSystem::SHADER_PROGRAM_NONE);
         queueDraw();
     }
+
+    // Synchronise the button state, if necessary
+    auto* toolbar = static_cast<wxToolBar*>(_mainPanel->FindWindow("RenderPreviewRenderModeToolbar"));
+    auto* textureButton = getToolBarToolByLabel(toolbar, "texturedModeButton");
+    auto* lightingButton = getToolBarToolByLabel(toolbar, "lightingModeButton");
+
+    if (!enabled && !textureButton->IsToggled())
+    {
+        toolbar->ToggleTool(textureButton->GetId(), true);
+    }
+    else if (enabled && !lightingButton->IsToggled())
+    {
+        toolbar->ToggleTool(lightingButton->GetId(), true);
+    }
 }
 
 const scene::GraphPtr& RenderPreview::getScene()
@@ -284,21 +299,6 @@ void RenderPreview::associateRenderSystem()
     }
 }
 
-Matrix4 RenderPreview::getProjectionMatrix(float near_z, float far_z, float fieldOfView, int width, int height)
-{
-    const float half_width = near_z * tan(degrees_to_radians(fieldOfView * 0.5f));
-    const float half_height = half_width * (static_cast<float>(height) / static_cast<float>(width));
-
-    return Matrix4::getProjectionForFrustum(
-        -half_width,
-        half_width,
-        -half_height,
-        half_height,
-        near_z,
-        far_z
-    );
-}
-
 const Matrix4& RenderPreview::getModelViewMatrix()
 {
     return _modelView;
@@ -322,11 +322,11 @@ Matrix4 RenderPreview::calculateModelViewMatrix()
     // 1. Move the view point away from the model (translate by -viewOrigin)
     // 2. Rotate the view using Euler angles (rotating about two specific axes)
     // 3. Apply the radiant2openGL matrix which basically rotates the model 90 degrees
-    //    around two axes. This is needed to resemble the view of a human (who - with 
+    //    around two axes. This is needed to resemble the view of a human (who - with
     //    pitch, yaw and roll angles equal to zero - looks parallel to the xy plane,
     //    and not down the negative z axis like openGL would.
 
-    // Translate the model by viewOrigin (with the later inverse() call this will 
+    // Translate the model by viewOrigin (with the later inverse() call this will
     // be the first applied translation by -viewOrigin)
     Matrix4 modelview = Matrix4::getTranslation(_viewOrigin);
 
@@ -428,6 +428,14 @@ bool RenderPreview::drawPreview()
     if (!_initialised)
     {
         initialisePreview();
+
+        // Since we shouldn't call the virtual canDrawGrid() in the constructor
+        // adjust the tool bar here.
+        if (!canDrawGrid())
+        {
+            auto* utilToolbar = findNamedObject<wxToolBar>(_mainPanel, "RenderPreviewUtilToolbar");
+            utilToolbar->DeleteTool(getToolBarToolByLabel(utilToolbar, "gridButton")->GetId());
+        }
     }
 
     util::ScopedBoolLock lock(_renderingInProgress);
@@ -435,7 +443,7 @@ bool RenderPreview::drawPreview()
     if (!_glFont)
     {
         auto fontSize = registry::getValue<int>(RKEY_RENDERPREVIEW_FONTSIZE);
-        auto fontStyle = registry::getValue<std::string>(RKEY_RENDERPREVIEW_FONTSTYLE) == "Sans" ? 
+        auto fontStyle = registry::getValue<std::string>(RKEY_RENDERPREVIEW_FONTSTYLE) == "Sans" ?
             IGLFont::Style::Sans : IGLFont::Style::Mono;
         _glFont = GlobalOpenGL().getFont(fontStyle, fontSize);
     }
@@ -444,7 +452,7 @@ bool RenderPreview::drawPreview()
 
     // Set up the render and clear the drawing area in any case
     glDepthMask(GL_TRUE);
-    
+
     if (getLightingModeEnabled())
     {
         glClearColor(0.0f, 0.0f, 0.0f, 1.0f);
@@ -465,7 +473,7 @@ bool RenderPreview::drawPreview()
 	}
 
     // Set up the camera
-    Matrix4 projection = getProjectionMatrix(0.1f, 10000, PREVIEW_FOV, _previewWidth, _previewHeight);
+    Matrix4 projection = camera::calculateProjectionMatrix(0.1f, 10000, PREVIEW_FOV, _previewWidth, _previewHeight);
 
     // Keep the modelview matrix in the volumetest class up to date
     _volumeTest.setModelView(getModelViewMatrix());
@@ -478,13 +486,13 @@ bool RenderPreview::drawPreview()
 	glMatrixMode(GL_MODELVIEW);
 	glLoadMatrixd(_volumeTest.GetModelview());
 
-	if (_renderGrid)
+	if (_renderGrid && canDrawGrid())
 	{
 		drawGrid();
 	}
 
 	// Front-end render phase, collect OpenGLRenderable objects from the scene
-    render::CamRenderer renderer(_volumeTest);
+    render::CamRenderer renderer(_volumeTest, _shaders);
     render::SceneRenderWalker sceneWalker(renderer, _volumeTest);
     getScene()->foreachVisibleNodeInVolume(_volumeTest, sceneWalker);
 
@@ -492,7 +500,7 @@ bool RenderPreview::drawPreview()
 
     // Launch the back end rendering
     renderer.submitToShaders();
-    _renderSystem->render(flags, _volumeTest.GetModelview(), projection);
+    _renderSystem->render(flags, _volumeTest.GetModelview(), projection, _viewOrigin);
 
     // Give subclasses an opportunity to render their own on-screen stuff
     onPostRender();
@@ -508,15 +516,15 @@ void RenderPreview::renderWireFrame()
     RenderStateFlags flags = getRenderFlagsWireframe();
 
     // Set up the camera
-    Matrix4 projection = getProjectionMatrix(0.1f, 10000, PREVIEW_FOV, _previewWidth, _previewHeight);
+    Matrix4 projection = camera::calculateProjectionMatrix(0.1f, 10000, PREVIEW_FOV, _previewWidth, _previewHeight);
 
     // Front-end render phase, collect OpenGLRenderable objects from the scene
-    render::CamRenderer renderer(_volumeTest);
+    render::CamRenderer renderer(_volumeTest, _shaders);
     render::SceneRenderWalker sceneWalker(renderer, _volumeTest);
     getScene()->foreachVisibleNodeInVolume(_volumeTest, sceneWalker);
 
     // Launch the back end rendering
-    _renderSystem->render(flags, _volumeTest.GetModelview(), projection);
+    _renderSystem->render(flags, _volumeTest.GetModelview(), projection, _viewOrigin);
 }
 
 void RenderPreview::onGLMouseClick(wxMouseEvent& ev)
@@ -605,6 +613,11 @@ AABB RenderPreview::getSceneBounds()
     return AABB(Vector3(0,0,0), Vector3(64,64,64));
 }
 
+Vector3 RenderPreview::getGridOrigin()
+{
+    return getSceneBounds().getOrigin();
+}
+
 void RenderPreview::resetModelRotation()
 {
     _modelRotation = Matrix4::getIdentity();
@@ -678,7 +691,7 @@ void RenderPreview::onPausePlaybackClick(wxCommandEvent& ev)
 	// Disable the button
 	wxToolBar* toolbar = findNamedObject<wxToolBar>(_mainPanel, "RenderPreviewAnimToolbar");
 	toolbar->EnableTool(getToolBarToolByLabel(toolbar, "pauseTimeButton")->GetId(), false);
-	
+
 	if (_timer.IsRunning())
     {
         _timer.Stop();
@@ -729,6 +742,11 @@ void RenderPreview::onSizeAllocate(wxSizeEvent& ev)
     _previewHeight = ev.GetSize().GetHeight();
 }
 
+bool RenderPreview::canDrawGrid()
+{
+    return true;
+}
+
 void RenderPreview::drawGrid()
 {
 	static float GRID_MAX_DIM = 512.0f;
@@ -744,8 +762,8 @@ void RenderPreview::drawGrid()
 
     glPushMatrix();
 
-    auto sceneCenter = getSceneBounds().getOrigin();
-    glTranslated(sceneCenter.x(), sceneCenter.y(), sceneCenter.z());
+    auto gridOrigin = getGridOrigin();
+    glTranslated(gridOrigin.x(), gridOrigin.y(), gridOrigin.z());
 
 	glBegin(GL_LINES);
 
