@@ -2,11 +2,10 @@
 
 #include "i18n.h"
 #include "imainframe.h"
-#include "iuimanager.h"
 
 #include "ReadableEditorDialog.h"
 
-#include <wx/artprov.h>
+#include "wxutil/Bitmap.h"
 #include <wx/sizer.h>
 
 namespace ui
@@ -28,8 +27,8 @@ XDataSelector::XDataSelector(const XData::StringVectorMap& files, ReadableEditor
 	_files(files),
 	_editorDialog(editorDialog)
 {
-	_xdataIcon.CopyFromBitmap(wxArtProvider::GetBitmap(GlobalUIManager().ArtIdPrefix() + XDATA_ICON));
-	_folderIcon.CopyFromBitmap(wxArtProvider::GetBitmap(GlobalUIManager().ArtIdPrefix() + FOLDER_ICON));
+	_xdataIcon.CopyFromBitmap(wxutil::GetLocalBitmap(XDATA_ICON));
+	_folderIcon.CopyFromBitmap(wxutil::GetLocalBitmap(FOLDER_ICON));
 
 	fillTree();
 
