@@ -8,9 +8,9 @@
 #include "itextstream.h"
 #include "iradiant.h"
 #include "ieventmanager.h"
-#include "iuimanager.h"
 #include "imainframe.h"
 #include "igroupdialog.h"
+#include "iundo.h"
 
 #include "interfaces/MathInterface.h"
 #include "interfaces/RegistryInterface.h"
@@ -134,7 +134,6 @@ void ScriptingSystem::initialise()
 
 void ScriptingSystem::runScriptFile(const cmd::ArgumentList& args)
 {
-	// Start the test script
 	if (args.empty()) return;
 
 	executeScriptFile(args[0].getString());
@@ -142,7 +141,6 @@ void ScriptingSystem::runScriptFile(const cmd::ArgumentList& args)
 
 void ScriptingSystem::runScriptCommand(const cmd::ArgumentList& args) 
 {
-	// Start the test script
 	if (args.empty()) return;
 
 	executeCommand(args[0].getString());
@@ -172,6 +170,8 @@ void ScriptingSystem::executeCommand(const std::string& name)
 		return;
 	}
 
+    UndoableCommand cmd("runScriptCommand " + name);
+
 	// Execute the script file behind this command
 	executeScriptFile(found->second->getFilename(), true);
 }
