@@ -20,7 +20,7 @@ void MathInterface::registerInterface(py::module& scope, py::dict& globals)
 	py::class_<Vector3> vec3(scope, "Vector3");
 	vec3.def(py::init<double, double, double>());
 	vec3.def(py::init<const Vector3&>());
-	
+
 	// greebo: Pick the correct overload - this is hard to read, but it is necessary
 	vec3.def("x", static_cast<double& (Vector3::*)()>(&Vector3::x), py::return_value_policy::reference);
 	vec3.def("y", static_cast<double& (Vector3::*)()>(&Vector3::y), py::return_value_policy::reference);
@@ -29,30 +29,23 @@ void MathInterface::registerInterface(py::module& scope, py::dict& globals)
 	vec3.def("getLengthSquared", &Vector3::getLengthSquared);
 	vec3.def("getNormalised", &Vector3::getNormalised);
 	vec3.def("normalise", &Vector3::normalise);
-	vec3.def("getInversed", &Vector3::getInversed);
-	vec3.def("dot", &Vector3::dot<double>);
-	vec3.def("angle", &Vector3::angle<double>);
-	vec3.def("crossProduct", &Vector3::crossProduct<double>);
-	vec3.def("max", &Vector3::max);
-	vec3.def("min", &Vector3::min);
-	vec3.def("isParallel", &Vector3::isParallel<double>);
+	vec3.def("dot", &Vector3::dot);
+	vec3.def("angle", &Vector3::angle);
+	vec3.def("crossProduct", &Vector3::cross);
 	// Most important operators
 	vec3.def(py::self + py::self);		// __add__
 	vec3.def(py::self - py::self);		// __sub__
 	vec3.def(py::self += py::self);		// __iadd__
 	vec3.def(py::self -= py::self);		// __isub__
 	vec3.def(py::self < py::self);	// __lt__
-	vec3.def("__repr__", [](const Vector3& vec) 
-	{ 
-		return "(" + string::to_string(vec.x()) + " " + string::to_string(vec.y()) + 
+	vec3.def("__repr__", [](const Vector3& vec)
+	{
+		return "(" + string::to_string(vec.x()) + " " + string::to_string(vec.y()) +
 			" " + string::to_string(vec.z()) + ")";
 	});
 
-	// Register Vertex3f, which extends Vector3
-	py::class_<Vertex3f, Vector3> vertex3f(scope, "Vertex3f");
-	vertex3f.def(py::init<>());
-	vertex3f.def(py::init<const Vector3&>());
-	vertex3f.def(py::init<double, double, double>());
+	// Register Vertex3f, which is an alias of Vector3
+    scope.add_object("Vertex3f", vec3);
 
 	// Add the Vector2 class
 	py::class_<Vector2> vec2(scope, "Vector2");
@@ -97,7 +90,7 @@ void MathInterface::registerInterface(py::module& scope, py::dict& globals)
 	vec4.def(py::self -= py::self);
 	vec4.def("__repr__", [](const Vector4& vec)
 	{
-		return "(" + string::to_string(vec.x()) + " " + string::to_string(vec.y()) + " " + 
+		return "(" + string::to_string(vec.x()) + " " + string::to_string(vec.y()) + " " +
 			string::to_string(vec.z()) + " " + string::to_string(vec.w()) + ")";
 	});
 
