@@ -73,6 +73,7 @@ CamWnd::CamWnd(wxWindow* parent) :
     _view(true),
     _camera(GlobalCameraManager().createCamera(_view, std::bind(&CamWnd::requestRedraw, this, std::placeholders::_1))),
     _drawing(false),
+    _updateRequested(false),
     _wxGLWidget(new wxutil::GLWidget(_mainWxWidget, std::bind(&CamWnd::onRender, this), "CamWnd")),
     _timer(this),
     _timerLock(false),
@@ -85,6 +86,7 @@ CamWnd::CamWnd(wxWindow* parent) :
 {
     Bind(wxEVT_TIMER, &CamWnd::onFrame, this, _timer.GetId());
     Bind(wxEVT_TIMER, &CamWnd::onFreeMoveTimer, this, _freeMoveTimer.GetId());
+    _wxGLWidget->Bind(wxEVT_IDLE, &CamWnd::onIdle, this);
 
     setFarClipPlaneDistance(calculateFarPlaneDistance(getCameraSettings()->cubicScale()));
     setFarClipPlaneEnabled(getCameraSettings()->farClipEnabled());
@@ -137,6 +139,7 @@ void CamWnd::constructToolbar()
     const wxToolBarToolBase* flatShadeBtn = getToolBarToolByLabel(_camToolbar, "flatShadeBtn");
     const wxToolBarToolBase* texturedBtn = getToolBarToolByLabel(_camToolbar, "texturedBtn");
     const wxToolBarToolBase* lightingBtn = getToolBarToolByLabel(_camToolbar, "lightingBtn");
+    const wxToolBarToolBase* gridButton = getToolBarToolByLabel(_camToolbar, "drawGridButton");
 
     if (!GlobalRenderSystem().shaderProgramsAvailable())
     {
@@ -156,6 +159,9 @@ void CamWnd::constructToolbar()
     _mainWxWidget->GetParent()->Bind(wxEVT_COMMAND_TOOL_CLICKED,&CamWnd::onRenderModeButtonsChanged, this, flatShadeBtn->GetId());
     _mainWxWidget->GetParent()->Bind(wxEVT_COMMAND_TOOL_CLICKED, &CamWnd::onRenderModeButtonsChanged, this, texturedBtn->GetId());
     _mainWxWidget->GetParent()->Bind(wxEVT_COMMAND_TOOL_CLICKED, &CamWnd::onRenderModeButtonsChanged, this, lightingBtn->GetId());
+    
+    auto toggleCameraGridEvent = GlobalEventManager().findEvent("ToggleCameraGrid");
+    toggleCameraGridEvent->connectToolItem(gridButton);
 
     // Far clip buttons.
     _farClipInID = getToolID(_camToolbar, "clipPlaneInButton");
@@ -236,6 +242,10 @@ void CamWnd::constructGUIComponents()
 
 CamWnd::~CamWnd()
 {
+    const wxToolBarToolBase* gridButton = getToolBarToolByLabel(_camToolbar, "drawGridButton");
+    auto toggleCameraGridEvent = GlobalEventManager().findEvent("ToggleCameraGrid");
+    toggleCameraGridEvent->disconnectToolItem(gridButton);
+
     // Stop the timer, it might still fire even during shutdown
     _timer.Stop();
 
@@ -588,6 +598,49 @@ void CamWnd::ensureFont()
     }
 }
 
+void CamWnd::drawGrid()
+{
+    static double GRID_MAX_DIM = 2048;
+    auto GRID_STEP = getCameraSettings()->gridSpacing();
+
+    glDisable(GL_TEXTURE_2D);
+    glDisable(GL_TEXTURE_1D);
+    glEnable(GL_DEPTH_TEST);
+    glDisable(GL_BLEND);
+
+    glLineWidth(1);
+    glColor3f(0.5f, 0.5f, 0.5f);
+
+    glPushMatrix();
+
+    // The grid is following the camera, but will always be in the z = 0 plane
+    auto origin = _camera->getCameraOrigin().getSnapped(GRID_STEP);
+    glTranslated(origin.x(), origin.y(), 0);
+
+    glBegin(GL_LINES);
+
+    for (double x = -GRID_MAX_DIM; x <= GRID_MAX_DIM; x += GRID_STEP)
+    {
+        Vector3 start(x, -GRID_MAX_DIM, 0);
+        Vector3 end(x, GRID_MAX_DIM, 0);
+
+        Vector3 start2(GRID_MAX_DIM, x, 0);
+        Vector3 end2(-GRID_MAX_DIM, x, 0);
+
+        glVertex2dv(start);
+        glVertex2dv(end);
+
+        glVertex2dv(start2);
+        glVertex2dv(end2);
+    }
+
+    glEnd();
+
+    glPopMatrix();
+
+    glDisable(GL_DEPTH_TEST);
+}
+
 void CamWnd::Cam_Draw()
 {
     wxSize glSize = _wxGLWidget->GetSize();
@@ -659,6 +712,10 @@ void CamWnd::Cam_Draw()
         glEnable(GL_LIGHT0);
     }
 
+    if (getCameraSettings()->gridEnabled() && getCameraSettings()->getRenderMode() != RENDER_MODE_LIGHTING)
+    {
+        drawGrid();
+    }
 
     // Set the allowed render flags for this view
     unsigned int allowedRenderFlags = RENDER_DEPTHTEST
@@ -722,8 +779,7 @@ void CamWnd::Cam_Draw()
     // Main scene render
     {
         // Front end (renderable collection from scene)
-        render::CamRenderer renderer(_view, _primitiveHighlightShader.get(),
-                                     _faceHighlightShader.get());
+        render::CamRenderer renderer(_view, _shaders);
         render::RenderableCollectionWalker::CollectRenderablesInScene(renderer, _view);
 
         // Accumulate render statistics
@@ -902,22 +958,34 @@ camera::ICameraView& CamWnd::getCamera()
 
 void CamWnd::captureStates()
 {
-    _faceHighlightShader = GlobalRenderSystem().capture("$CAM_HIGHLIGHT");
-    _primitiveHighlightShader = GlobalRenderSystem().capture("$CAM_OVERLAY");
+    _shaders.faceHighlightShader = GlobalRenderSystem().capture("$CAM_HIGHLIGHT");
+    _shaders.primitiveHighlightShader = GlobalRenderSystem().capture("$CAM_OVERLAY");
+    _shaders.mergeActionShaderAdd = GlobalRenderSystem().capture("$MERGE_ACTION_ADD");
+    _shaders.mergeActionShaderChange = GlobalRenderSystem().capture("$MERGE_ACTION_CHANGE");
+    _shaders.mergeActionShaderRemove = GlobalRenderSystem().capture("$MERGE_ACTION_REMOVE");
+    _shaders.mergeActionShaderConflict = GlobalRenderSystem().capture("$MERGE_ACTION_CONFLICT");
 }
 
-void CamWnd::releaseStates() {
-    _faceHighlightShader = ShaderPtr();
-    _primitiveHighlightShader = ShaderPtr();
+void CamWnd::releaseStates() 
+{
+    _shaders.faceHighlightShader.reset();
+    _shaders.primitiveHighlightShader.reset();
+    _shaders.mergeActionShaderAdd.reset();
+    _shaders.mergeActionShaderChange.reset();
+    _shaders.mergeActionShaderRemove.reset();
+    _shaders.mergeActionShaderConflict.reset();
 }
 
 void CamWnd::queueDraw()
 {
-    if (_drawing)
-    {
-        return;
-    }
+    _updateRequested = true;
+}
 
+void CamWnd::onIdle(wxIdleEvent& ev)
+{
+    if (!_updateRequested) return;
+
+    _updateRequested = false;
     _wxGLWidget->Refresh(false);
 }
 
@@ -1323,8 +1391,7 @@ void CamWnd::rotateRightDiscrete()
 
 // -------------------------------------------------------------------------------
 
-ShaderPtr CamWnd::_faceHighlightShader;
-ShaderPtr CamWnd::_primitiveHighlightShader;
+render::CamRenderer::HighlightShaders CamWnd::_shaders;
 int CamWnd::_maxId = 0;
 
 } // namespace
