@@ -26,6 +26,7 @@
 #include <sigc++/connection.h>
 #include "tools/CameraMouseToolEvent.h"
 #include "render/RenderStatistics.h"
+#include "render/CamRenderer.h"
 
 const int CAMWND_MINSIZE_X = 240;
 const int CAMWND_MINSIZE_Y = 200;
@@ -61,14 +62,16 @@ class CamWnd :
     // The contained camera
     camera::ICameraView::Ptr _camera;
 
-    static ShaderPtr _faceHighlightShader;
-    static ShaderPtr _primitiveHighlightShader;
+    static render::CamRenderer::HighlightShaders _shaders;
 
     wxutil::FreezePointer _freezePointer;
 
     // Is true during an active drawing process
     bool _drawing;
 
+    // Update of this window in the next idle event loop
+    bool _updateRequested;
+
     // The GL widget
     wxutil::GLWidget* _wxGLWidget;
 
@@ -219,6 +222,7 @@ private:
     void Cam_Draw();
     bool onRender();
     void drawTime();
+    void drawGrid();
     void requestRedraw(bool force);
 
     // Motion and ICameraView related
@@ -241,6 +245,7 @@ private:
 
     void onFrame(wxTimerEvent& ev);
     void onFreeMoveTimer(wxTimerEvent& ev);
+    void onIdle(wxIdleEvent& ev);
 
     void handleFreeMovement(float timePassed);
     void setFreeMoveFlags(unsigned int mask);
