@@ -1,10 +1,13 @@
 #include "UserInterfaceModule.h"
 
+#include <sigc++/functors/ptr_fun.h>
+
 #include "i18n.h"
 #include "ilayer.h"
 #include "ifilter.h"
 #include "ientity.h"
 #include "imru.h"
+#include "imap.h"
 #include "ibrush.h"
 #include "ipatch.h"
 #include "iorthocontextmenu.h"
@@ -51,7 +54,8 @@
 #include "modelexport/ExportAsModelDialog.h"
 #include "modelexport/ExportCollisionModelDialog.h"
 #include "ui/filters/FilterOrthoContextMenuItem.h"
-#include "uimanager/colourscheme/ColourSchemeEditor.h"
+#include "ui/colourscheme/ColourSchemeEditor.h"
+#include "ui/animationpreview/MD5AnimationViewer.h"
 #include "ui/layers/CreateLayerDialog.h"
 #include "ui/patch/PatchCreateDialog.h"
 #include "ui/patch/BulgePatchDialog.h"
@@ -60,6 +64,8 @@
 #include "ui/brush/FindBrush.h"
 #include "ui/mousetool/RegistrationHelper.h"
 #include "ui/mapselector/MapSelector.h"
+#include "ui/merge/MergeControlDialog.h"
+#include "ui/PointFileChooser.h"
 
 #include <wx/version.h>
 
@@ -81,27 +87,25 @@ namespace
 
 const std::string& UserInterfaceModule::getName() const
 {
-	static std::string _name("UserInterfaceModule");
+	static std::string _name(MODULE_USERINTERFACE);
 	return _name;
 }
 
 const StringSet& UserInterfaceModule::getDependencies() const
 {
-	static StringSet _dependencies;
-
-	if (_dependencies.empty())
-	{
-		_dependencies.insert(MODULE_LAYERS);
-		_dependencies.insert(MODULE_ORTHOCONTEXTMENU);
-		_dependencies.insert(MODULE_UIMANAGER);
-		_dependencies.insert(MODULE_FILTERSYSTEM);
-		_dependencies.insert(MODULE_ENTITY);
-		_dependencies.insert(MODULE_EVENTMANAGER);
-		_dependencies.insert(MODULE_RADIANT_CORE);
-		_dependencies.insert(MODULE_MRU_MANAGER);
-		_dependencies.insert(MODULE_MAINFRAME);
-		_dependencies.insert(MODULE_MOUSETOOLMANAGER);
-	}
+    static StringSet _dependencies
+    {
+        MODULE_LAYERS,
+        MODULE_ORTHOCONTEXTMENU,
+        MODULE_FILTERSYSTEM,
+        MODULE_ENTITY,
+        MODULE_EVENTMANAGER,
+        MODULE_RADIANT_CORE,
+        MODULE_MRU_MANAGER,
+        MODULE_MAINFRAME,
+        MODULE_MOUSETOOLMANAGER,
+        MODULE_MAP
+    };
 
 	return _dependencies;
 }
@@ -132,19 +136,19 @@ void UserInterfaceModule::initialiseModule(const IApplicationContext& ctx)
 
 	// Add the orthocontext menu's layer actions
 	GlobalOrthoContextMenu().addItem(
-		std::make_shared<LayerOrthoContextMenuItem>(_(ADD_TO_LAYER_TEXT), 
+		std::make_shared<LayerOrthoContextMenuItem>(_(ADD_TO_LAYER_TEXT),
 			LayerOrthoContextMenuItem::AddToLayer),
 		IOrthoContextMenu::SECTION_LAYER
 	);
 
 	GlobalOrthoContextMenu().addItem(
-		std::make_shared<LayerOrthoContextMenuItem>(_(MOVE_TO_LAYER_TEXT), 
+		std::make_shared<LayerOrthoContextMenuItem>(_(MOVE_TO_LAYER_TEXT),
 			LayerOrthoContextMenuItem::MoveToLayer),
 		IOrthoContextMenu::SECTION_LAYER
 	);
 
 	GlobalOrthoContextMenu().addItem(
-		std::make_shared<LayerOrthoContextMenuItem>(_(REMOVE_FROM_LAYER_TEXT), 
+		std::make_shared<LayerOrthoContextMenuItem>(_(REMOVE_FROM_LAYER_TEXT),
 			LayerOrthoContextMenuItem::RemoveFromLayer),
 		IOrthoContextMenu::SECTION_LAYER
 	);
@@ -209,14 +213,19 @@ void UserInterfaceModule::initialiseModule(const IApplicationContext& ctx)
 	SelectionSetToolmenu::Init();
 
 	_mruMenu.reset(new MRUMenu);
-	_shaderClipboardStatus.reset(new ShaderClipboardStatus);
-	_editStopwatchStatus.reset(new EditingStopwatchStatus);
+	_shaderClipboardStatus.reset(new statusbar::ShaderClipboardStatus);
+	_editStopwatchStatus.reset(new statusbar::EditingStopwatchStatus);
+    _mapStatisticsStatus.reset(new statusbar::MapStatistics);
 	_manipulatorToggle.reset(new ManipulatorToggle);
 	_selectionModeToggle.reset(new SelectionModeToggle);
 
 	MouseToolRegistrationHelper::RegisterTools();
 
 	wxTheApp->Bind(DISPATCH_EVENT, &UserInterfaceModule::onDispatchEvent, this);
+
+    _mapEditModeChangedConn = GlobalMapModule().signal_editModeChanged().connect(
+        sigc::ptr_fun(&MergeControlDialog::OnMapEditModeChanged)
+    );
 }
 
 void UserInterfaceModule::shutdownModule()
@@ -229,6 +238,7 @@ void UserInterfaceModule::shutdownModule()
 
 	_coloursUpdatedConn.disconnect();
 	_entitySettingsConn.disconnect();
+    _mapEditModeChangedConn.disconnect();
 
 	_longOperationHandler.reset();
 	_mapFileProgressHandler.reset();
@@ -236,10 +246,11 @@ void UserInterfaceModule::shutdownModule()
 	_fileSelectionRequestHandler.reset();
 	_autoSaveRequestHandler.reset();
 	_shaderClipboardStatus.reset();
+    _mapStatisticsStatus.reset();
 	_editStopwatchStatus.reset();
 	_manipulatorToggle.reset();
 	_selectionModeToggle.reset();
-	
+
 	_mruMenu.reset();
 }
 
@@ -265,17 +276,17 @@ void UserInterfaceModule::HandleNotificationMessage(radiant::NotificationMessage
 	switch (msg.getType())
 	{
 	case radiant::NotificationMessage::Information:
-		wxutil::Messagebox::Show(msg.hasTitle() ? msg.getTitle() : _("Notification"), 
+		wxutil::Messagebox::Show(msg.hasTitle() ? msg.getTitle() : _("Notification"),
 			msg.getMessage(), IDialog::MessageType::MESSAGE_CONFIRM, parentWindow);
 		break;
 
 	case radiant::NotificationMessage::Warning:
-		wxutil::Messagebox::Show(msg.hasTitle() ? msg.getTitle() : _("Warning"), 
+		wxutil::Messagebox::Show(msg.hasTitle() ? msg.getTitle() : _("Warning"),
 			msg.getMessage(), IDialog::MessageType::MESSAGE_WARNING, parentWindow);
 		break;
 
 	case radiant::NotificationMessage::Error:
-		wxutil::Messagebox::Show(msg.hasTitle() ? msg.getTitle() : _("Error"), 
+		wxutil::Messagebox::Show(msg.hasTitle() ? msg.getTitle() : _("Error"),
 			msg.getMessage(), IDialog::MessageType::MESSAGE_ERROR, parentWindow);
 		break;
 	};
@@ -300,9 +311,9 @@ void UserInterfaceModule::initialiseEntitySettings()
 	applyPatchVertexColours();
 
 	_coloursUpdatedConn = ColourSchemeEditor::signal_ColoursChanged().connect(
-		[this]() { 
-			applyEntityVertexColours(); 
-			applyBrushVertexColours(); 
+		[this]() {
+			applyEntityVertexColours();
+			applyBrushVertexColours();
 			applyPatchVertexColours();
 		}
 	);
@@ -343,7 +354,7 @@ void UserInterfaceModule::refreshShadersCmd(const cmd::ArgumentList& args)
 	// Disable screen updates for the scope of this function
 	auto blocker = GlobalMainFrame().getScopedScreenUpdateBlocker(_("Processing..."), _("Loading Shaders"));
 
-	// Reload the Shadersystem, this will also trigger an 
+	// Reload the Shadersystem, this will also trigger an
 	// OpenGLRenderSystem unrealise/realise sequence as the rendersystem
 	// is attached to this class as Observer
 	// We can't do this refresh() operation in a thread it seems due to context binding
@@ -363,10 +374,16 @@ void UserInterfaceModule::registerUICommands()
 	GlobalCommandSystem().addCommand("ToggleLightInspector", LightInspector::toggleInspector);
 	GlobalCommandSystem().addCommand("SurfaceInspector", SurfaceInspector::toggle);
 	GlobalCommandSystem().addCommand("PatchInspector", PatchInspector::toggle);
+	GlobalCommandSystem().addCommand("MergeControlDialog", MergeControlDialog::ShowDialog);
 	GlobalCommandSystem().addCommand("OverlayDialog", OverlayDialog::toggle);
 	GlobalCommandSystem().addCommand("TransformDialog", TransformDialog::toggle);
+    GlobalCommandSystem().addCommand("ChooseAndTogglePointfile",
+                                     [](const cmd::ArgumentList&)
+                                     { PointFileChooser::chooseAndToggle(); });
 
-	GlobalCommandSystem().addCommand("FindBrush", FindBrushDialog::Show);
+    GlobalCommandSystem().addCommand("FindBrush", FindBrushDialog::Show);
+    GlobalCommandSystem().addCommand("AnimationPreview", MD5AnimationViewer::Show);
+    GlobalCommandSystem().addCommand("EditColourScheme", ColourSchemeEditor::DisplayDialog);
 
 	GlobalCommandSystem().addCommand("MapInfo", MapInfoDialog::ShowDialog);
 	GlobalCommandSystem().addCommand("MouseToolMappingDialog", ToolMappingDialog::ShowDialog);
