@@ -3,7 +3,7 @@
 #include <sigc++/connection.h>
 #include <wx/event.h>
 
-#include "imodule.h"
+#include "iuserinterface.h"
 #include "iorthocontextmenu.h"
 #include "icommandsystem.h"
 
@@ -14,8 +14,9 @@
 #include "MapFileProgressHandler.h"
 #include "ManipulatorToggle.h"
 #include "SelectionModeToggle.h"
-#include "shaderclipboard/ShaderClipboardStatus.h"
+#include "statusbar/ShaderClipboardStatus.h"
 #include "statusbar/EditingStopwatchStatus.h"
+#include "statusbar/MapStatistics.h"
 #include "messages/CommandExecutionFailed.h"
 #include "messages/TextureChanged.h"
 #include "messages/NotificationMessage.h"
@@ -34,7 +35,7 @@ namespace ui
  */
 class UserInterfaceModule :
 	public wxEvtHandler,
-	public RegisterableModule
+	public IUserInterfaceModule
 {
 private:
 	std::unique_ptr<LongRunningOperationHandler> _longOperationHandler;
@@ -42,13 +43,15 @@ private:
 	std::unique_ptr<AutoSaveRequestHandler> _autoSaveRequestHandler;
 	std::unique_ptr<FileSelectionRequestHandler> _fileSelectionRequestHandler;
 	std::unique_ptr<FileOverwriteConfirmationHandler> _fileOverwriteConfirmationHandler;
-	std::unique_ptr<ShaderClipboardStatus> _shaderClipboardStatus;
-	std::unique_ptr<EditingStopwatchStatus> _editStopwatchStatus;
+	std::unique_ptr<statusbar::ShaderClipboardStatus> _shaderClipboardStatus;
+	std::unique_ptr<statusbar::EditingStopwatchStatus> _editStopwatchStatus;
+	std::unique_ptr<statusbar::MapStatistics> _mapStatisticsStatus;
 	std::unique_ptr<ManipulatorToggle> _manipulatorToggle;
 	std::unique_ptr<SelectionModeToggle> _selectionModeToggle;
 
 	sigc::connection _entitySettingsConn;
 	sigc::connection _coloursUpdatedConn;
+    sigc::connection _mapEditModeChangedConn;
 
 	std::size_t _execFailedListener;
 	std::size_t _textureChangedListener;
@@ -66,7 +69,7 @@ public:
 	// Runs the specified action in the UI thread 
 	// this happens when the application has a chance to, usually during event processing
 	// This method is safe to be called from any thread.
-	void dispatch(const std::function<void()>& action);
+	void dispatch(const std::function<void()>& action) override;
 
 private:
 	void registerUICommands();
