@@ -44,11 +44,13 @@ public:
 
 	// Called during AAS module initialisation
 	static void Init();
-    static void OnRadiantStartup();
 
 	static AasControlDialog& Instance();
 
 private:
+    static void onMainFrameConstructed();
+    void onMainFrameShuttingDown();
+
 	static AasControlDialogPtr& InstancePtr();
 
 	// TransientWindow events
@@ -56,7 +58,6 @@ private:
 
 	void populateWindow();
     void createButtons();
-    void onRadiantShutdown();
 	void clearControls();
 
 	void onMapEvent(IMap::MapEvent ev);
