@@ -3,7 +3,6 @@
 
 #include "i18n.h"
 #include "imainframe.h"
-#include "iuimanager.h"
 #include "ieclass.h"
 #include "igame.h"
 #include "ientity.h"
@@ -13,7 +12,7 @@
 #include <wx/sizer.h>
 #include <wx/button.h>
 #include <wx/panel.h>
-#include <wx/artprov.h>
+#include "wxutil/Bitmap.h"
 #include <wx/textctrl.h>
 
 #include <map>
@@ -69,12 +68,12 @@ void AddPropertyDialog::setupTreeView()
 
 	_treeView->AssociateModel(_treeStore.get());
 
-	_treeView->Connect(wxEVT_DATAVIEW_SELECTION_CHANGED, 
+	_treeView->Connect(wxEVT_DATAVIEW_SELECTION_CHANGED,
 		wxDataViewEventHandler(AddPropertyDialog::_onSelectionChanged), NULL, this);
 
 	// Display name column with icon
-	_treeView->AppendIconTextColumn(_("Property"), 
-		_columns.displayName.getColumnIndex(), wxDATAVIEW_CELL_INERT, 
+	_treeView->AppendIconTextColumn(_("Property"),
+		_columns.displayName.getColumnIndex(), wxDATAVIEW_CELL_INERT,
 		wxCOL_WIDTH_AUTOSIZE, wxALIGN_NOT, wxDATAVIEW_COL_SORTABLE);
 
 	// Use the TreeModel's full string search function
@@ -115,7 +114,7 @@ public:
 	{ }
 
 	// Required visit function
-	void operator() (const EntityClassAttribute& attr)
+	void operator() (const EntityClassAttribute& attr, bool)
 	{
 		// greebo: Only add the property if it hasn't been set directly on the entity itself.
 		if (!_entity->getKeyValue(attr.getName()).empty() && !_entity->isInherited(attr.getName()))
@@ -148,7 +147,7 @@ public:
 void AddPropertyDialog::populateTreeView()
 {
 	wxIcon folderIcon;
-	folderIcon.CopyFromBitmap(wxArtProvider::GetBitmap(GlobalUIManager().ArtIdPrefix() + FOLDER_ICON));
+	folderIcon.CopyFromBitmap(wxutil::GetLocalBitmap(FOLDER_ICON));
 
 	// DEF-DEFINED PROPERTIES
 	{
@@ -172,10 +171,10 @@ void AddPropertyDialog::populateTreeView()
 		// Use a CustomPropertyAdder class to visit the entityclass and add all
 		// custom properties from it
 		CustomPropertyAdder adder(_entity, _treeStore, _columns, defRoot.getItem());
-		_entity->getEntityClass()->forEachClassAttribute(std::ref(adder));
+		_entity->getEntityClass()->forEachAttribute(std::ref(adder));
 	}
 
-	// REGISTRY (GAME FILE) DEFINED PROPERTIES 
+	// REGISTRY (GAME FILE) DEFINED PROPERTIES
 
 	// Ask the XML registry for the list of properties
     game::IGamePtr currentGame = GlobalGameManager().currentGame();
