@@ -7,8 +7,8 @@
 #include "ieclass.h"
 #include "iregistry.h"
 #include "ieventmanager.h"
-#include "iuimanager.h"
 #include "igame.h"
+#include "imap.h"
 #include "iundo.h"
 #include "igroupdialog.h"
 #include "imainframe.h"
@@ -18,9 +18,11 @@
 #include "selectionlib.h"
 #include "scene/SelectionIndex.h"
 #include "scenelib.h"
+#include "eclass.h"
 #include "wxutil/dialog/MessageBox.h"
 #include "wxutil/menu/IconTextMenuItem.h"
 #include "wxutil/dataview/TreeModel.h"
+#include "wxutil/dataview/TreeViewItemStyle.h"
 #include "xmlutil/Document.h"
 
 #include <map>
@@ -34,7 +36,7 @@
 #include <wx/splitter.h>
 #include <wx/textctrl.h>
 #include <wx/bmpbuttn.h>
-#include <wx/artprov.h>
+#include "wxutil/Bitmap.h"
 
 #include <functional>
 #include "string/replace.h"
@@ -62,13 +64,18 @@ EntityInspector::EntityInspector() :
     _showHelpColumnCheckbox(nullptr),
     _primitiveNumLabel(nullptr),
     _keyValueTreeView(nullptr),
+    _booleanColumn(nullptr),
+    _valueColumn(nullptr),
+    _oldValueColumn(nullptr),
+    _newValueColumn(nullptr),
     _keyEntry(nullptr),
-    _valEntry(nullptr)
+    _valEntry(nullptr),
+    _setButton(nullptr)
 {}
 
 void EntityInspector::construct()
 {
-    _emptyIcon.CopyFromBitmap(wxArtProvider::GetBitmap(GlobalUIManager().ArtIdPrefix() + "empty.png"));
+    _emptyIcon.CopyFromBitmap(wxutil::GetLocalBitmap("empty.png"));
     wxASSERT(_emptyIcon.IsOk());
 
     wxFrame* temporaryParent = new wxFrame(NULL, wxID_ANY, "");
@@ -84,7 +91,7 @@ void EntityInspector::construct()
     _showInheritedCheckbox->Bind(wxEVT_CHECKBOX, [this](wxCommandEvent& ev) {
         handleShowInheritedChanged();
     });
-    
+
     _showHelpColumnCheckbox = new wxCheckBox(_mainWidget, wxID_ANY, _("Show help"));
     _showHelpColumnCheckbox->Bind(wxEVT_CHECKBOX, [this](wxCommandEvent& ev) {
         handleShowHelpTextChanged();
@@ -105,10 +112,10 @@ void EntityInspector::construct()
     _paned->SplitHorizontally(createTreeViewPane(_paned), createPropertyEditorPane(_paned));
     _panedPosition.connect(_paned);
 
-    _helpText = new wxTextCtrl(_mainWidget, wxID_ANY, "", 
+    _helpText = new wxTextCtrl(_mainWidget, wxID_ANY, "",
         wxDefaultPosition, wxDefaultSize, wxTE_LEFT | wxTE_MULTILINE | wxTE_READONLY | wxTE_WORDWRAP);
     _helpText->SetMinClientSize(wxSize(-1, 60));
-    
+
     _mainWidget->GetSizer()->Add(optionsHBox, 0, wxEXPAND | wxALL, 3);
     _mainWidget->GetSizer()->Add(_paned, 1, wxEXPAND);
     _mainWidget->GetSizer()->Add(_helpText, 0, wxEXPAND);
@@ -121,7 +128,7 @@ void EntityInspector::construct()
 
     handleShowInheritedChanged();
     handleShowHelpTextChanged();
-    
+
     // Reload the information from the registry
     restoreSettings();
 
@@ -134,6 +141,10 @@ void EntityInspector::construct()
     // Register self to the SelectionSystem to get notified upon selection
     // changes.
     GlobalSelectionSystem().addObserver(this);
+    
+    _defsReloadedHandler = GlobalEntityClassManager().defsReloadedSignal().connect(
+        sigc::mem_fun(this, &EntityInspector::onDefsReloaded)
+    );
 
     // Observe the Undo system for undo/redo operations, to refresh the
     // keyvalues when this happens
@@ -142,6 +153,10 @@ void EntityInspector::construct()
     _redoHandler = GlobalUndoSystem().signal_postRedo().connect(
         sigc::mem_fun(this, &EntityInspector::onUndoRedoOperation));
 
+    _mapEditModeChangedHandler = GlobalMapModule().signal_editModeChanged().connect(
+        sigc::mem_fun(this, &EntityInspector::onMapEditModeChanged)
+    );
+
     // initialise the properties
     loadPropertyMap();
 }
@@ -160,6 +175,13 @@ void EntityInspector::restoreSettings()
     }
 }
 
+void EntityInspector::onMapEditModeChanged(IMap::EditMode mode)
+{
+    // Clear the selection to not hold any references to merge nodes
+    changeSelectedEntity(scene::INodePtr(), scene::INodePtr());
+    requestIdleCallback();
+}
+
 // Entity::Observer implementation
 
 void EntityInspector::onKeyInsert(const std::string& key,
@@ -176,7 +198,7 @@ void EntityInspector::onKeyChange(const std::string& key,
 
     // Check if we already have an iter for this key (i.e. this is a
     // modification).
-    TreeIterMap::const_iterator i = _keyValueIterMap.find(key);
+    auto i = _keyValueIterMap.find(key);
 
     if (i != _keyValueIterMap.end())
     {
@@ -213,8 +235,34 @@ void EntityInspector::onKeyChange(const std::string& key,
     // Set the values for the row
     wxutil::TreeModel::Row row(keyValueIter, *_kvStore);
 
-    wxDataViewItemAttr black;
-    black.SetColour(wxColor(0,0,0));
+    wxDataViewItemAttr style;
+
+    // Check if this key is affected by a merge operation
+    auto action = _mergeActions.find(key);
+
+    if (action != _mergeActions.end())
+    {
+        switch (action->second->getType())
+        {
+        case scene::merge::ActionType::AddKeyValue:
+            wxutil::TreeViewItemStyle::ApplyKeyValueAddedStyle(style);
+            break;
+        case scene::merge::ActionType::ChangeKeyValue:
+            wxutil::TreeViewItemStyle::ApplyKeyValueChangedStyle(style);
+            break;
+        case scene::merge::ActionType::RemoveKeyValue:
+            wxutil::TreeViewItemStyle::ApplyKeyValueRemovedStyle(style);
+            break;
+        }
+    }
+
+    // Conflicting actions get a special render style
+    auto conflict = _conflictActions.find(key);
+
+    if (conflict != _conflictActions.end() && conflict->second->getResolution() == scene::merge::ResolutionType::Unresolved)
+    {
+        wxutil::TreeViewItemStyle::ApplyKeyValueConflictStyle(style);
+    }
 
     wxIcon icon;
     icon.CopyFromBitmap(parms.type.empty() ? _emptyIcon : PropertyEditorFactory::getBitmapFor(parms.type));
@@ -236,12 +284,41 @@ void EntityInspector::onKeyChange(const std::string& key,
         row[_columns.booleanValue].setEnabled(false);
     }
 
-    // Text colour
-    row[_columns.name] = black;
-    row[_columns.value] = black;
+    if (action != _mergeActions.end())
+    {
+        if (action->second->getType() == scene::merge::ActionType::AddKeyValue)
+        {
+            row[_columns.oldValue] = std::string(); // no old value to show
+            row[_columns.oldValue] = style;
+        }
+        else
+        {
+            wxDataViewItemAttr oldAttr = style;
+            wxutil::TreeViewItemStyle::SetStrikethrough(oldAttr, true);
+            row[_columns.oldValue] = value;
+            row[_columns.oldValue] = oldAttr;
+        }
+
+        row[_columns.newValue] = action->second->getValue();
+
+        wxDataViewItemAttr newAttr = style;
+        newAttr.SetBold(true);
+
+        row[_columns.newValue] = newAttr;
+    }
+    else
+    {
+        row[_columns.oldValue] = std::string();
+        row[_columns.newValue] = std::string();
+    }
+
+    // Apply background style to all other columns
+    row[_columns.name] = style;
+    row[_columns.value] = style;
+    row[_columns.booleanValue] = style;
 
     row[_columns.isInherited] = false;
-    row[_columns.hasHelpText] = hasDescription;
+    row[_columns.hasHelpText] = key == "classname" || hasDescription; // classname always has a description
 
     if (added)
     {
@@ -325,6 +402,22 @@ void EntityInspector::createContextMenu()
         std::bind(&EntityInspector::_onPasteKey, this),
         std::bind(&EntityInspector::_testPasteKey, this)
     );
+
+    _contextMenu->addSeparator();
+
+    _contextMenu->addItem(
+        new wxutil::StockIconTextMenuItem(_("Accept selected Changes"), wxART_TICK_MARK),
+        std::bind(&EntityInspector::_onAcceptMergeAction, this),
+        std::bind(&EntityInspector::_testAcceptMergeAction, this),
+        [] { return GlobalMapModule().getEditMode() == IMap::EditMode::Merge; }
+    );
+
+    _contextMenu->addItem(
+        new wxutil::StockIconTextMenuItem(_("Reject selected Changes"), wxART_UNDO),
+        std::bind(&EntityInspector::_onRejectMergeAction, this),
+        std::bind(&EntityInspector::_testRejectMergeAction, this),
+        [] { return GlobalMapModule().getEditMode() == IMap::EditMode::Merge; }
+    );
 }
 
 void EntityInspector::onMainFrameConstructed()
@@ -344,8 +437,13 @@ void EntityInspector::onMainFrameConstructed()
 
 void EntityInspector::onMainFrameShuttingDown()
 {
+    _mergeActions.clear();
+    _conflictActions.clear();
+
+    _mapEditModeChangedHandler.disconnect();
     _undoHandler.disconnect();
     _redoHandler.disconnect();
+    _defsReloadedHandler.disconnect();
 
     // Remove all previously stored pane information
     _panedPosition.saveToPath(RKEY_PANE_STATE);
@@ -356,9 +454,19 @@ void EntityInspector::onMainFrameShuttingDown()
 }
 
 void EntityInspector::onUndoRedoOperation()
+{
+    refresh();
+}
+
+void EntityInspector::onDefsReloaded()
+{
+    refresh();
+}
+
+void EntityInspector::refresh()
 {
     // Clear the previous entity (detaches this class as observer)
-    changeSelectedEntity(nullptr);
+    changeSelectedEntity(scene::INodePtr(), scene::INodePtr());
 
     // Now rescan the selection and update the stores
     requestIdleCallback();
@@ -372,19 +480,17 @@ const std::string& EntityInspector::getName() const
 
 const StringSet& EntityInspector::getDependencies() const
 {
-    static StringSet _dependencies;
-
-    if (_dependencies.empty())
+    static StringSet _dependencies
     {
-        _dependencies.insert(MODULE_XMLREGISTRY);
-        _dependencies.insert(MODULE_UIMANAGER);
-        _dependencies.insert(MODULE_SELECTIONSYSTEM);
-        _dependencies.insert(MODULE_UNDOSYSTEM);
-        _dependencies.insert(MODULE_GAMEMANAGER);
-        _dependencies.insert(MODULE_COMMANDSYSTEM);
-        _dependencies.insert(MODULE_EVENTMANAGER);
-        _dependencies.insert(MODULE_MAINFRAME);
-    }
+        MODULE_XMLREGISTRY,
+        MODULE_GROUPDIALOG,
+        MODULE_SELECTIONSYSTEM,
+        MODULE_UNDOSYSTEM,
+        MODULE_GAMEMANAGER,
+        MODULE_COMMANDSYSTEM,
+        MODULE_EVENTMANAGER,
+        MODULE_MAINFRAME
+    };
 
     return _dependencies;
 }
@@ -445,35 +551,40 @@ wxWindow* EntityInspector::createTreeViewPane(wxWindow* parent)
     _kvStore = new wxutil::TreeModel(_columns, true); // this is a list model
 
     _keyValueTreeView = wxutil::TreeView::CreateWithModel(treeViewPanel, _kvStore.get(), wxDV_MULTIPLE);
+    _keyValueTreeView->EnableAutoColumnWidthFix(true);
 
     // Search in both name and value columns
     _keyValueTreeView->AddSearchColumn(_columns.name);
     _keyValueTreeView->AddSearchColumn(_columns.value);
 
     // Add the checkbox for boolean properties
-    _keyValueTreeView->AppendToggleColumn("", _columns.booleanValue.getColumnIndex(),
+    _booleanColumn = _keyValueTreeView->AppendToggleColumn("", _columns.booleanValue.getColumnIndex(),
         wxDATAVIEW_CELL_ACTIVATABLE, wxCOL_WIDTH_AUTOSIZE, wxALIGN_NOT);
 
     // Create the Property column (has an icon)
-    _keyValueTreeView->AppendIconTextColumn(_("Property"), 
-        _columns.name.getColumnIndex(), wxDATAVIEW_CELL_INERT, 
+    _keyValueTreeView->AppendIconTextColumn(_("Property"),
+        _columns.name.getColumnIndex(), wxDATAVIEW_CELL_INERT,
         wxCOL_WIDTH_AUTOSIZE, wxALIGN_NOT, wxDATAVIEW_COL_RESIZABLE | wxDATAVIEW_COL_SORTABLE);
 
     // Create the value column
-    _keyValueTreeView->AppendTextColumn(_("Value"), 
-        _columns.value.getColumnIndex(), wxDATAVIEW_CELL_INERT, 
+    _valueColumn = _keyValueTreeView->AppendTextColumn(_("Value"),
+        _columns.value.getColumnIndex(), wxDATAVIEW_CELL_INERT,
+        wxCOL_WIDTH_AUTOSIZE, wxALIGN_NOT, wxDATAVIEW_COL_RESIZABLE | wxDATAVIEW_COL_SORTABLE);
+
+    _oldValueColumn = _keyValueTreeView->AppendTextColumn(_("Old Value"),
+        _columns.oldValue.getColumnIndex(), wxDATAVIEW_CELL_INERT,
+        wxCOL_WIDTH_AUTOSIZE, wxALIGN_NOT, wxDATAVIEW_COL_RESIZABLE | wxDATAVIEW_COL_SORTABLE);
+    _newValueColumn = _keyValueTreeView->AppendTextColumn(_("New Value"),
+        _columns.newValue.getColumnIndex(), wxDATAVIEW_CELL_INERT,
         wxCOL_WIDTH_AUTOSIZE, wxALIGN_NOT, wxDATAVIEW_COL_RESIZABLE | wxDATAVIEW_COL_SORTABLE);
 
     // Used to update the help text
-    _keyValueTreeView->Connect(wxEVT_DATAVIEW_SELECTION_CHANGED, 
-        wxDataViewEventHandler(EntityInspector::_onTreeViewSelectionChanged), NULL, this);
-    _keyValueTreeView->Connect(wxEVT_DATAVIEW_ITEM_CONTEXT_MENU, 
-        wxDataViewEventHandler(EntityInspector::_onContextMenu), NULL, this);
+    _keyValueTreeView->Bind(wxEVT_DATAVIEW_SELECTION_CHANGED, &EntityInspector::_onTreeViewSelectionChanged, this);
+    _keyValueTreeView->Bind(wxEVT_DATAVIEW_ITEM_CONTEXT_MENU, &EntityInspector::_onContextMenu, this);
 
     // When the toggle column is clicked to check/uncheck the box, the model's column value
     // is directly changed by the wxWidgets event handlers. On model value change, this event is fired afterwards
-    _keyValueTreeView->Connect(wxEVT_DATAVIEW_ITEM_VALUE_CHANGED,
-        wxDataViewEventHandler(EntityInspector::_onDataViewItemChanged), NULL, this);
+    _keyValueTreeView->Bind(wxEVT_DATAVIEW_ITEM_VALUE_CHANGED, &EntityInspector::_onDataViewItemChanged, this);
 
     wxBoxSizer* buttonHbox = new wxBoxSizer(wxHORIZONTAL);
 
@@ -482,18 +593,18 @@ wxWindow* EntityInspector::createTreeViewPane(wxWindow* parent)
     _valEntry = new wxTextCtrl(treeViewPanel, wxID_ANY, "", wxDefaultPosition, wxDefaultSize, wxTE_PROCESS_ENTER);
 
     wxBitmap icon = wxArtProvider::GetBitmap(wxART_TICK_MARK, wxART_MENU);
-    wxBitmapButton* setButton = new wxBitmapButton(treeViewPanel, wxID_APPLY, icon);
+    _setButton = new wxBitmapButton(treeViewPanel, wxID_APPLY, icon);
 
     buttonHbox->Add(_valEntry, 1, wxEXPAND);
-    buttonHbox->Add(setButton, 0, wxEXPAND);
+    buttonHbox->Add(_setButton, 0, wxEXPAND);
 
     treeViewPanel->GetSizer()->Add(_keyValueTreeView, 1, wxEXPAND);
     treeViewPanel->GetSizer()->Add(_keyEntry, 0, wxEXPAND);
     treeViewPanel->GetSizer()->Add(buttonHbox, 0, wxEXPAND);
 
-    setButton->Connect(wxEVT_BUTTON, wxCommandEventHandler(EntityInspector::_onSetProperty), NULL, this);
-    _keyEntry->Connect(wxEVT_TEXT_ENTER, wxCommandEventHandler(EntityInspector::_onEntryActivate), NULL, this);
-    _valEntry->Connect(wxEVT_TEXT_ENTER, wxCommandEventHandler(EntityInspector::_onEntryActivate), NULL, this);
+    _setButton->Bind(wxEVT_BUTTON, &EntityInspector::_onSetProperty, this);
+    _keyEntry->Bind(wxEVT_TEXT_ENTER, &EntityInspector::_onEntryActivate, this);
+    _valEntry->Bind(wxEVT_TEXT_ENTER, &EntityInspector::_onEntryActivate, this);
 
     return treeViewPanel;
 }
@@ -555,21 +666,43 @@ bool EntityInspector::getListSelectionBool(const wxutil::TreeModel::Column& col)
     return row[col].getBool();
 }
 
+bool EntityInspector::canUpdateEntity()
+{
+    return GlobalMapModule().getEditMode() != IMap::EditMode::Merge;
+}
+
 // Redraw the GUI elements
 void EntityInspector::updateGUIElements()
 {
     // Update from selection system
     getEntityFromSelectionSystem();
 
+    auto entityCanBeUpdated = canUpdateEntity();
+
+    auto isMergeMode = GlobalMapModule().getEditMode() == IMap::EditMode::Merge;
+    _oldValueColumn->SetHidden(!isMergeMode);
+    _newValueColumn->SetHidden(!isMergeMode);
+
+    // Set the value column back to the default AUTO setting
+    _valueColumn->SetWidth(wxCOL_WIDTH_AUTOSIZE);
+
     if (!_selectedEntity.expired())
     {
-        _editorFrame->Enable(true);
+        _editorFrame->Enable(entityCanBeUpdated);
         _keyValueTreeView->Enable(true);
         _showInheritedCheckbox->Enable(true);
         _showHelpColumnCheckbox->Enable(true);
+        _keyEntry->Enable(entityCanBeUpdated);
+        _valEntry->Enable(entityCanBeUpdated);
+        _setButton->Enable(entityCanBeUpdated);
+        _booleanColumn->GetRenderer()->SetMode(entityCanBeUpdated ? wxDATAVIEW_CELL_ACTIVATABLE : wxDATAVIEW_CELL_INERT);
 
+        if (!entityCanBeUpdated)
+        {
+            _currentPropertyEditor.reset();
+        }
 		// Update the target entity on any active property editor (#5092)
-		if (_currentPropertyEditor)
+        else if (_currentPropertyEditor)
 		{
 			auto newEntity = Node_getEntity(_selectedEntity.lock());
 			assert(newEntity != nullptr);
@@ -682,7 +815,7 @@ void EntityInspector::_onAddKey()
 
 bool EntityInspector::_testAddKey()
 {
-    return !_selectedEntity.expired();
+    return !_selectedEntity.expired() && canUpdateEntity();
 }
 
 void EntityInspector::_onDeleteKey()
@@ -731,7 +864,7 @@ void EntityInspector::_onCopyKey()
 
     if (selectedItems.Count() == 0) return;
 
-    _clipBoard.clear();
+    _clipboard.clear();
 
     for (const wxDataViewItem& item : selectedItems)
     {
@@ -742,7 +875,7 @@ void EntityInspector::_onCopyKey()
         std::string key = iconAndName.GetText().ToStdString();
         std::string value = row[_columns.value];
 
-        _clipBoard.emplace_back(key, value);
+        _clipboard.emplace_back(key, value);
     }
 }
 
@@ -761,7 +894,7 @@ void EntityInspector::_onCutKey()
     assert(!_selectedEntity.expired());
     Entity* selectedEntity = Node_getEntity(_selectedEntity.lock());
 
-    _clipBoard.clear();
+    _clipboard.clear();
     std::unique_ptr<UndoableCommand> cmd;
 
     for (const wxDataViewItem& item : selectedItems)
@@ -782,7 +915,7 @@ void EntityInspector::_onCutKey()
         auto key = iconAndName.GetText().ToStdString();
         auto value = row[_columns.value];
 
-        _clipBoard.emplace_back(key, value);
+        _clipboard.emplace_back(key, value);
 
         // Clear the key after copying
         selectedEntity->setKeyValue(key, "");
@@ -805,8 +938,8 @@ bool EntityInspector::isItemDeletable(const wxutil::TreeModel::Row& row)
 
 bool EntityInspector::_testNonEmptyAndDeletableSelection()
 {
-    if (_selectedEntity.expired()) return false;
-    
+    if (_selectedEntity.expired() || !canUpdateEntity()) return false;
+
     wxDataViewItemArray selectedItems;
     _keyValueTreeView->GetSelections(selectedItems);
 
@@ -834,7 +967,7 @@ void EntityInspector::_onPasteKey()
     // greebo: Instantiate a scoped object to make this operation undoable
     UndoableCommand command("entitySetProperties");
 
-    for (const KeyValuePair& kv : _clipBoard)
+    for (const KeyValuePair& kv : _clipboard)
     {
         // skip empty entries
         if (kv.first.empty() || kv.second.empty()) continue;
@@ -853,7 +986,141 @@ bool EntityInspector::_testPasteKey()
     }
 
     // Return true if the clipboard contains data
-    return !_clipBoard.empty();
+    return !_clipboard.empty() && canUpdateEntity();
+}
+
+void EntityInspector::_onAcceptMergeAction()
+{
+    wxDataViewItemArray selectedItems;
+    _keyValueTreeView->GetSelections(selectedItems);
+
+    for (const wxDataViewItem& item : selectedItems)
+    {
+        wxutil::TreeModel::Row row(item, *_kvStore);
+
+        auto key = row[_columns.name].getString().ToStdString();
+
+        auto conflict = _conflictActions.find(key);
+
+        if (conflict != _conflictActions.end())
+        {
+            conflict->second->setResolution(scene::merge::ResolutionType::ApplySourceChange);
+        }
+    }
+
+    // We perform a full refresh of the view
+    changeSelectedEntity(scene::INodePtr(), scene::INodePtr());
+    getEntityFromSelectionSystem();
+}
+
+void EntityInspector::_onRejectMergeAction()
+{
+    wxDataViewItemArray selectedItems;
+    _keyValueTreeView->GetSelections(selectedItems);
+
+    for (const wxDataViewItem& item : selectedItems)
+    {
+        wxutil::TreeModel::Row row(item, *_kvStore);
+
+        auto key = row[_columns.name].getString().ToStdString();
+        
+        auto conflict = _conflictActions.find(key);
+
+        if (conflict != _conflictActions.end())
+        {
+            conflict->second->setResolution(scene::merge::ResolutionType::RejectSourceChange);
+            // Deactivate the conflict action itself too, such that the node can be removed once the last action is gone
+            conflict->second->deactivate();
+        }
+
+        auto action = _mergeActions.find(key);
+        
+        if (action != _mergeActions.end())
+        {
+            action->second->deactivate();
+        }
+    }
+
+    // Check if the merge node is now completely empty, then remove it from the scene
+    // A single entity must be selected
+    if (GlobalSelectionSystem().countSelected() == 1)
+    {
+        auto selectedNode = GlobalSelectionSystem().ultimateSelected();
+
+        if (selectedNode && selectedNode->getNodeType() == scene::INode::Type::MergeAction)
+        {
+            auto mergeNode = std::dynamic_pointer_cast<scene::IMergeActionNode>(selectedNode);
+            assert(mergeNode);
+
+            if (!mergeNode->hasActiveActions())
+            {
+                // Remove this node from the scene, it's empty now
+                scene::removeNodeFromParent(mergeNode);
+            }
+        }
+    }
+
+    // We perform a full refresh of the view
+    changeSelectedEntity(scene::INodePtr(), scene::INodePtr());
+    getEntityFromSelectionSystem();
+}
+
+bool EntityInspector::_testAcceptMergeAction()
+{
+    if (GlobalMapModule().getEditMode() != IMap::EditMode::Merge)
+    {
+        return false;
+    }
+
+    wxDataViewItemArray selectedItems;
+    _keyValueTreeView->GetSelections(selectedItems);
+
+    for (const wxDataViewItem& item : selectedItems)
+    {
+        wxutil::TreeModel::Row row(item, *_kvStore);
+
+        if (isItemAffecedByMergeConflict(row))
+        {
+            return true;
+        }
+    }
+
+    return false;
+}
+
+bool EntityInspector::_testRejectMergeAction()
+{
+    if (GlobalMapModule().getEditMode() != IMap::EditMode::Merge)
+    {
+        return false;
+    }
+
+    wxDataViewItemArray selectedItems;
+    _keyValueTreeView->GetSelections(selectedItems);
+
+    for (const wxDataViewItem& item : selectedItems)
+    {
+        wxutil::TreeModel::Row row(item, *_kvStore);
+
+        if (isItemAffecedByMergeOperation(row))
+        {
+            return true;
+        }
+    }
+
+    return false;
+}
+
+bool EntityInspector::isItemAffecedByMergeConflict(const wxutil::TreeModel::Row& row)
+{
+    auto key = row[_columns.name].getString().ToStdString();
+    return _mergeActions.count(key) > 0 && _conflictActions.count(key) > 0;
+}
+
+bool EntityInspector::isItemAffecedByMergeOperation(const wxutil::TreeModel::Row& row)
+{
+    auto key = row[_columns.name].getString().ToStdString();
+    return _mergeActions.count(key) > 0 || _conflictActions.count(key) > 0;
 }
 
 // wxWidget callbacks
@@ -865,7 +1132,7 @@ void EntityInspector::_onContextMenu(wxDataViewEvent& ev)
 
 void EntityInspector::_onDataViewItemChanged(wxDataViewEvent& ev)
 {
-    if (ev.GetDataViewColumn() != nullptr && 
+    if (ev.GetDataViewColumn() != nullptr &&
         static_cast<int>(ev.GetDataViewColumn()->GetModelColumn()) == _columns.booleanValue.getColumnIndex())
     {
         // Model value in the boolean column has changed, this means
@@ -879,7 +1146,7 @@ void EntityInspector::_onDataViewItemChanged(wxDataViewEvent& ev)
 
         UndoableCommand cmd("entitySetProperty");
         applyKeyValueToSelection(key, updatedValue ? "1" : "0");
-        
+
         // Check if the property was an inherited one.
         // The applyKeyValue function produced a non-inherited entry
         // which should be visible once we're done
@@ -960,16 +1227,23 @@ void EntityInspector::updateHelpText(const wxutil::TreeModel::Row& row)
     // Get the key pointed at
     bool hasHelp = row[_columns.hasHelpText].getBool();
 
-    if (hasHelp)
+    if (!hasHelp) return;
+    
+    std::string key = getSelectedKey();
+
+    assert(!_selectedEntity.expired());
+    auto* selectedEntity = Node_getEntity(_selectedEntity.lock());
+
+    auto eclass = selectedEntity->getEntityClass();
+    assert(eclass);
+
+    if (key == "classname")
+    {
+        // #5621: Show the editor_usage string when the classname is selected
+        _helpText->SetValue(eclass::getUsage(*eclass));
+    }
+    else
     {
-        std::string key = getSelectedKey();
-
-        assert(!_selectedEntity.expired());
-        Entity* selectedEntity = Node_getEntity(_selectedEntity.lock());
-
-        IEntityClassConstPtr eclass = selectedEntity->getEntityClass();
-        assert(eclass != NULL);
-
         // Find the attribute on the eclass, that's where the descriptions are defined
         const EntityClassAttribute& attr = eclass->getAttribute(key);
 
@@ -1012,30 +1286,6 @@ void EntityInspector::_onTreeViewSelectionChanged(wxDataViewEvent& ev)
         std::string key = getSelectedKey();
         std::string value = getListSelection(_columns.value);
 
-        // Get the type for this key if it exists, and the options
-        PropertyParms parms = getPropertyParmsForKey(key);
-
-        Entity* selectedEntity = Node_getEntity(_selectedEntity.lock());
-
-        // If the type was not found, also try looking on the entity class
-        if (parms.type.empty())
-        {
-            IEntityClassConstPtr eclass = selectedEntity->getEntityClass();
-            parms.type = eclass->getAttribute(key).getType();
-        }
-
-        // Construct and add a new PropertyEditor
-        _currentPropertyEditor = PropertyEditorFactory::create(_editorFrame,
-            parms.type, selectedEntity, key, parms.options);
-
-        if (_currentPropertyEditor)
-        {
-            // Don't use wxEXPAND to allow for horizontal centering, just add a 6 pixel border
-            // Using wxALIGN_CENTER_HORIZONTAL will position the property editor's panel in the middle
-            _editorFrame->GetSizer()->Add(_currentPropertyEditor->getWidget(), 1, wxALIGN_CENTER_HORIZONTAL | wxALL, 6);
-            _editorFrame->GetSizer()->Layout();
-        }
-
         // Update key and value entry boxes, but only if there is a key value. If
         // there is no selection we do not clear the boxes, to allow keyval copying
         // between entities.
@@ -1044,6 +1294,34 @@ void EntityInspector::_onTreeViewSelectionChanged(wxDataViewEvent& ev)
             _keyEntry->SetValue(key);
             _valEntry->SetValue(value);
         }
+
+        // Update property editor, unless we're in merge mode
+        if (canUpdateEntity())
+        {
+            // Get the type for this key if it exists, and the options
+            PropertyParms parms = getPropertyParmsForKey(key);
+
+            Entity* selectedEntity = Node_getEntity(_selectedEntity.lock());
+
+            // If the type was not found, also try looking on the entity class
+            if (parms.type.empty())
+            {
+                IEntityClassConstPtr eclass = selectedEntity->getEntityClass();
+                parms.type = eclass->getAttribute(key).getType();
+            }
+
+            // Construct and add a new PropertyEditor
+            _currentPropertyEditor = PropertyEditorFactory::create(_editorFrame,
+                parms.type, selectedEntity, key, parms.options);
+
+            if (_currentPropertyEditor)
+            {
+                // Don't use wxEXPAND to allow for horizontal centering, just add a 6 pixel border
+                // Using wxALIGN_CENTER_HORIZONTAL will position the property editor's panel in the middle
+                _editorFrame->GetSizer()->Add(_currentPropertyEditor->getWidget(), 1, wxALIGN_CENTER_HORIZONTAL | wxALL, 6);
+                _editorFrame->GetSizer()->Layout();
+            }
+        }
     }
     else if (selectedItems.Count() > 1)
     {
@@ -1089,9 +1367,9 @@ void EntityInspector::addClassAttribute(const EntityClassAttribute& a)
         wxutil::TreeModel::Row row = _kvStore->AddItem();
 
         wxDataViewItemAttr grey;
-        grey.SetColour(wxColor(112, 112, 112));
+        grey.SetItalic(true);
 
-        row[_columns.name] = wxVariant(wxDataViewIconText(a.getName(), _emptyIcon)); 
+        row[_columns.name] = wxVariant(wxDataViewIconText(a.getName(), _emptyIcon));
         row[_columns.value] = a.getValue();
 
         // Inherited values have an inactive checkbox, so assign a false value and disable
@@ -1107,6 +1385,8 @@ void EntityInspector::addClassAttribute(const EntityClassAttribute& a)
 
         row[_columns.name] = grey;
         row[_columns.value] = grey;
+        row[_columns.oldValue] = std::string();
+        row[_columns.newValue] = std::string();
 
         row[_columns.isInherited] = true;
         row[_columns.hasHelpText] = hasDescription;
@@ -1140,8 +1420,8 @@ void EntityInspector::addClassProperties()
     }
 
     // Visit the entity class
-    eclass->forEachClassAttribute(
-        std::bind(&EntityInspector::addClassAttribute, this, std::placeholders::_1)
+    eclass->forEachAttribute(
+        [&](const EntityClassAttribute& a, bool) { addClassAttribute(a); }
     );
 }
 
@@ -1161,18 +1441,18 @@ void EntityInspector::getEntityFromSelectionSystem()
     // A single entity must be selected
     if (GlobalSelectionSystem().countSelected() != 1)
     {
-        changeSelectedEntity(scene::INodePtr());
+        changeSelectedEntity(scene::INodePtr(), scene::INodePtr());
         _primitiveNumLabel->SetLabelText("");
         return;
     }
 
-    scene::INodePtr selectedNode = GlobalSelectionSystem().ultimateSelected();
+    auto selectedNode = GlobalSelectionSystem().ultimateSelected();
 
     // The root node must not be selected (this can happen if Invert Selection is
     // activated with an empty scene, or by direct selection in the entity list).
     if (selectedNode->isRoot())
     {
-        changeSelectedEntity(scene::INodePtr());
+        changeSelectedEntity(scene::INodePtr(), selectedNode);
         _primitiveNumLabel->SetLabelText("");
         return;
     }
@@ -1184,7 +1464,7 @@ void EntityInspector::getEntityFromSelectionSystem()
     if (newSelectedEntity)
     {
         // Node was an entity, use this
-        changeSelectedEntity(selectedNode);
+        changeSelectedEntity(selectedNode, selectedNode);
 
         // Just set the entity number
         auto indices = scene::getNodeIndices(selectedNode);
@@ -1193,10 +1473,31 @@ void EntityInspector::getEntityFromSelectionSystem()
     }
     else
     {
+        // Check if this is a special merge node
+        if (selectedNode->getNodeType() == scene::INode::Type::MergeAction &&
+            GlobalMapModule().getEditMode() == IMap::EditMode::Merge)
+        {
+            auto mergeAction = std::dynamic_pointer_cast<scene::IMergeActionNode>(selectedNode);
+            assert(mergeAction);
+
+            if (mergeAction && Node_isEntity(mergeAction->getAffectedNode()))
+            {
+                // Use the entity of the merge node
+                changeSelectedEntity(mergeAction->getAffectedNode(), selectedNode);
+                _primitiveNumLabel->SetLabelText(_("Merge Preview"));
+                return;
+            }
+        }
+
         // Node was not an entity, try parent instead
         scene::INodePtr selectedNodeParent = selectedNode->getParent();
-        changeSelectedEntity(selectedNodeParent);
-        
+        changeSelectedEntity(selectedNodeParent, selectedNode);
+
+        if (!_selectedEntity.lock())
+        {
+            return;
+        }
+
         try
         {
             auto indices = scene::getNodeIndices(selectedNode);
@@ -1211,8 +1512,7 @@ void EntityInspector::getEntityFromSelectionSystem()
     }
 }
 
-// Change selected entity pointer
-void EntityInspector::changeSelectedEntity(const scene::INodePtr& newEntity)
+void EntityInspector::changeSelectedEntity(const scene::INodePtr& newEntity, const scene::INodePtr& selectedNode)
 {
     // Check what we need to do with the existing entity
     scene::INodePtr oldEntity = _selectedEntity.lock();
@@ -1237,19 +1537,24 @@ void EntityInspector::changeSelectedEntity(const scene::INodePtr& newEntity)
     // it has already been deleted (no disconnection necessary)
     _selectedEntity.reset();
 
-    // Clear the view. If the old entity has been destroyed before we had 
+    // Clear the view. If the old entity has been destroyed before we had
     // a chance to disconnect the list store might contain remnants
     _keyValueIterMap.clear();
     _kvStore->Clear();
+    _mergeActions.clear();
+    _conflictActions.clear();
 
     // Reset the sorting when changing entities
     _keyValueTreeView->ResetSortingOnAllColumns();
-    
+
     // Attach to new entity if it is non-NULL
-    if (newEntity)
+    if (newEntity && newEntity->getNodeType() == scene::INode::Type::Entity)
     {
         _selectedEntity = newEntity;
 
+        // Any possible merge actions go in first
+        handleMergeActions(selectedNode);
+
         // Attach as observer to fill the listview
         Node_getEntity(newEntity)->attachObserver(this);
 
@@ -1261,6 +1566,71 @@ void EntityInspector::changeSelectedEntity(const scene::INodePtr& newEntity)
     }
 }
 
+void EntityInspector::handleKeyValueMergeAction(const scene::merge::IEntityKeyValueMergeAction::Ptr& mergeAction)
+{
+    // Remember this action in the map, it will be used in onKeyChange()
+    _mergeActions[mergeAction->getKey()] = mergeAction;
+
+    auto selectedEntity = _selectedEntity.lock();
+
+    // Keys added by a merge operation won't be handled in onKeyChange(), so do this here
+    if (mergeAction->getType() == scene::merge::ActionType::AddKeyValue ||
+        (mergeAction->getType() == scene::merge::ActionType::ChangeKeyValue && selectedEntity && 
+         Node_getEntity(selectedEntity)->getKeyValue(mergeAction->getKey()).empty()))
+    {
+        onKeyChange(mergeAction->getKey(), mergeAction->getValue());
+    }
+}
+
+void EntityInspector::handleMergeActions(const scene::INodePtr& selectedNode)
+{
+    // Any possible merge actions go in first
+    if (GlobalMapModule().getEditMode() != IMap::EditMode::Merge || !selectedNode ||
+        selectedNode->getNodeType() != scene::INode::Type::MergeAction)
+    {
+        return;
+    }
+
+    auto mergeNode = std::dynamic_pointer_cast<scene::IMergeActionNode>(selectedNode);
+
+    if (!mergeNode) return;
+    
+    mergeNode->foreachMergeAction([&](const scene::merge::IMergeAction::Ptr& action)
+    {
+        if (!action->isActive()) return; // don't apply inactive actions to our view
+
+        auto entityKeyValueAction = std::dynamic_pointer_cast<scene::merge::IEntityKeyValueMergeAction>(action);
+
+        if (entityKeyValueAction)
+        {
+            handleKeyValueMergeAction(entityKeyValueAction);
+        }
+
+        auto conflictAction = std::dynamic_pointer_cast<scene::merge::IConflictResolutionAction>(action);
+
+        if (conflictAction)
+        {
+            bool conflictIsUnresolved = conflictAction->getResolution() == scene::merge::ResolutionType::Unresolved;
+
+            if (conflictIsUnresolved || conflictAction->getResolution() == scene::merge::ResolutionType::ApplySourceChange)
+            {
+                auto sourceAction = std::dynamic_pointer_cast<scene::merge::IEntityKeyValueMergeAction>(conflictAction->getSourceAction());
+
+                if (sourceAction)
+                {
+                    handleKeyValueMergeAction(sourceAction);
+
+                    // Remember the conflict action if it's not yet resolved
+                    if (conflictIsUnresolved)
+                    {
+                        _conflictActions[sourceAction->getKey()] = conflictAction;
+                    }
+                }
+            }
+        }
+    });
+}
+
 void EntityInspector::toggle(const cmd::ArgumentList& args)
 {
     GlobalGroupDialog().togglePage("entity");
