@@ -4,9 +4,9 @@
 #include "igroupdialog.h"
 #include "imap.h"
 #include "ieventmanager.h"
-#include "iuimanager.h"
 #include "ipreferencesystem.h"
 #include "ientityinspector.h"
+#include "ieclass.h"
 #include "iorthoview.h"
 #include "iregistry.h"
 #include "iradiant.h"
@@ -19,7 +19,7 @@
 #include "wxutil/MultiMonitor.h"
 
 #include "ui/mainframe/ScreenUpdateBlocker.h"
-#include "ui/mainframe/EmbeddedLayout.h"
+#include "ui/mainframe/AuiLayout.h"
 #include "ui/mainframe/TopLevelFrame.h"
 
 #include "module/StaticModule.h"
@@ -34,7 +34,7 @@
 #include <windows.h>
 #endif
 
-namespace 
+namespace
 {
 	const std::string RKEY_WINDOW_LAYOUT = "user/ui/mainFrame/windowLayout";
 	const std::string RKEY_WINDOW_STATE = "user/ui/mainFrame/window";
@@ -44,12 +44,13 @@ namespace
 	const std::string RKEY_ACTIVE_LAYOUT = "user/ui/mainFrame/activeLayout";
 }
 
-namespace ui 
+namespace ui
 {
 
 MainFrame::MainFrame() :
 	_topLevelWindow(NULL),
-	_screenUpdatesEnabled(false) // not enabled until constructed
+	_screenUpdatesEnabled(false), // not enabled until constructed
+    _defLoadingBlocksUpdates(false)
 {}
 
 // RegisterableModule implementation
@@ -71,6 +72,7 @@ const StringSet& MainFrame::getDependencies() const
 		_dependencies.insert(MODULE_EVENTMANAGER);
 		_dependencies.insert(MODULE_COMMANDSYSTEM);
 		_dependencies.insert(MODULE_ORTHOVIEWMANAGER);
+		_dependencies.insert(MODULE_ECLASSMANAGER);
 		_dependencies.insert(MODULE_MAP);
 	}
 
@@ -146,6 +148,17 @@ void MainFrame::initialiseModule(const IApplicationContext& ctx)
 		sigc::mem_fun(this, &MainFrame::updateTitle)
 	);
 
+    // When the eclass defs are in progress of being loaded, block all updates
+    GlobalEntityClassManager().defsLoadingSignal().connect(
+        [this]() { _defLoadingBlocksUpdates = true; }
+    );
+
+    GlobalEntityClassManager().defsLoadedSignal().connect([this]()
+        { 
+            _defLoadingBlocksUpdates = false;
+        }
+    );
+
 	// Subscribe for the post-module init event
 	module::GlobalModuleRegistry().signal_allModulesInitialised().connect(
 		sigc::mem_fun(this, &MainFrame::postModuleInitialisation));
@@ -240,7 +253,7 @@ void MainFrame::construct()
 
 	if (activeLayout.empty())
 	{
-		activeLayout = EMBEDDED_LAYOUT_NAME; // fall back to hardcoded layout
+		activeLayout = AUI_LAYOUT_NAME; // fall back to hardcoded layout
 	}
 
 	// Apply the layout
@@ -251,10 +264,10 @@ void MainFrame::construct()
 		// Layout is still empty, this is not good
 		rError() << "Could not restore layout " << activeLayout << std::endl;
 
-		if (activeLayout != EMBEDDED_LAYOUT_NAME)
+		if (activeLayout != AUI_LAYOUT_NAME)
 		{
 			// Try to fallback to floating layout
-			applyLayout(EMBEDDED_LAYOUT_NAME);
+			applyLayout(AUI_LAYOUT_NAME);
 		}
 	}
 
@@ -339,7 +352,7 @@ void MainFrame::onTopLevelFrameClose(wxCloseEvent& ev)
 			return;
 		}
     }
-    
+
     wxASSERT(wxTheApp->GetTopWindow() == _topLevelWindow);
 
     _topLevelWindow->Hide();
@@ -351,7 +364,7 @@ void MainFrame::onTopLevelFrameClose(wxCloseEvent& ev)
 	// Broadcast shutdown event
 	signal_MainFrameShuttingDown().emit();
 	signal_MainFrameShuttingDown().clear();
-	
+
     // Destroy the actual window
     _topLevelWindow->Destroy();
     _topLevelWindow = nullptr;
@@ -484,8 +497,9 @@ void MainFrame::saveWindowPosition()
 	}
 }
 
-bool MainFrame::screenUpdatesEnabled() {
-	return _screenUpdatesEnabled;
+bool MainFrame::screenUpdatesEnabled()
+{
+	return _screenUpdatesEnabled && !_defLoadingBlocksUpdates;
 }
 
 void MainFrame::enableScreenUpdates() {
@@ -558,7 +572,7 @@ std::string MainFrame::getCurrentLayout()
 	return (_currentLayout != NULL) ? _currentLayout->getName() : "";
 }
 
-IScopedScreenUpdateBlockerPtr MainFrame::getScopedScreenUpdateBlocker(const std::string& title, 
+IScopedScreenUpdateBlockerPtr MainFrame::getScopedScreenUpdateBlocker(const std::string& title,
 		const std::string& message, bool forceDisplay)
 {
 	return IScopedScreenUpdateBlockerPtr(new ScreenUpdateBlocker(title, message, forceDisplay));
