@@ -2,12 +2,13 @@
 
 #include "itextstream.h"
 #include "ieventmanager.h"
-#include "iuimanager.h"
 #include "imainframe.h"
+#include "igroupdialog.h"
 #include "icommandsystem.h"
 
 #include "module/StaticModule.h"
 
+#include "AuiLayout.h"
 #include "FloatingLayout.h"
 #include "SplitPaneLayout.h"
 #include "RegularLayout.h"
@@ -45,7 +46,7 @@ void MainFrameLayoutManager::registerLayout(
 	}
 }
 
-void MainFrameLayoutManager::registerCommands() 
+void MainFrameLayoutManager::registerCommands()
 {
 	// remove all commands beforehand
 	_commands.clear();
@@ -67,13 +68,12 @@ const std::string& MainFrameLayoutManager::getName() const
 
 const StringSet& MainFrameLayoutManager::getDependencies() const
 {
-	static StringSet _dependencies;
-
-	if (_dependencies.empty()) {
-		_dependencies.insert(MODULE_EVENTMANAGER);
-		_dependencies.insert(MODULE_UIMANAGER);
-		_dependencies.insert(MODULE_COMMANDSYSTEM);
-	}
+    static StringSet _dependencies
+    {
+        MODULE_EVENTMANAGER,
+        MODULE_GROUPDIALOG,
+        MODULE_COMMANDSYSTEM
+    };
 
 	return _dependencies;
 }
@@ -84,7 +84,7 @@ void MainFrameLayoutManager::initialiseModule(const IApplicationContext& ctx)
 
 	// Register the default layouts
 	registerLayout(EMBEDDED_LAYOUT_NAME, EmbeddedLayout::CreateInstance);
-
+	registerLayout(AUI_LAYOUT_NAME, AuiLayout::CreateInstance);
 	registerLayout(FLOATING_LAYOUT_NAME, FloatingLayout::CreateInstance);
 	registerLayout(SPLITPANE_LAYOUT_NAME, SplitPaneLayout::CreateInstance);
 	registerLayout(REGULAR_LAYOUT_NAME, RegularLayout::CreateRegularInstance);
