@@ -2,7 +2,7 @@
 
 #include "itextstream.h"
 #include "ieventmanager.h"
-#include "iuimanager.h"
+#include "imenumanager.h"
 #include "igroupdialog.h"
 #include "imainframe.h"
 #include "ientityinspector.h"
@@ -95,16 +95,16 @@ void RegularLayout::activate()
 	topLevelParent->Layout();
 
 	// Hide the camera toggle option for non-floating views
-    GlobalUIManager().getMenuManager().setVisibility("main/view/cameraview", false);
+    GlobalMenuManager().setVisibility("main/view/cameraview", false);
 	// Hide the console/texture browser toggles for non-floating/non-split views
-	GlobalUIManager().getMenuManager().setVisibility("main/view/textureBrowser", false);
+	GlobalMenuManager().setVisibility("main/view/textureBrowser", false);
 }
 
 void RegularLayout::deactivate()
 {
 	// Show the camera toggle option again
-    GlobalUIManager().getMenuManager().setVisibility("main/view/cameraview", true);
-	GlobalUIManager().getMenuManager().setVisibility("main/view/textureBrowser", true);
+    GlobalMenuManager().setVisibility("main/view/cameraview", true);
+	GlobalMenuManager().setVisibility("main/view/textureBrowser", true);
 
 	// Remove all previously stored pane information
 	GlobalRegistry().deleteXPath(RKEY_REGULAR_ROOT + "//pane");
