@@ -7,7 +7,6 @@
 
 #include "modelskin.h"
 #include "iradiant.h"
-#include "iuimanager.h"
 
 #include "wxutil/dialog/DialogBase.h"
 #include "wxutil/preview/ModelPreview.h"
@@ -112,6 +111,7 @@ private:
 	void onCancel(wxCommandEvent& ev);
 	void onReloadModels(wxCommandEvent& ev);
 	void onReloadSkins(wxCommandEvent& ev);
+    void onRescanFolders(wxCommandEvent& ev);
     void onTreeViewPopulationFinished(wxutil::ResourceTreeView::PopulationFinishedEvent& ev);
 
 	// Update the info table with information from the currently-selected model, and
