@@ -11,7 +11,6 @@
 #include "selectionlib.h"
 #include "ientity.h" // Node_getEntity()
 #include "iregistry.h"
-#include "iuimanager.h"
 #include "imainframe.h"
 
 #include "wxutil/dialog/MessageBox.h"
@@ -258,13 +257,13 @@ std::string OrthoContextMenu::getRegistryKeyWithDefault(
 
 void OrthoContextMenu::addEntity()
 {
-    UndoableCommand command("createEntity");
-
     // Display the chooser to select an entity classname
     std::string cName = wxutil::EntityClassChooser::chooseEntityClass();
 
     if (!cName.empty()) 
     {
+        UndoableCommand command("createEntity");
+
         // Create the entity. We might get an EntityCreationException if the
         // wrong number of brushes is selected.
         try
@@ -606,15 +605,12 @@ const std::string& OrthoContextMenu::getName() const
 
 const StringSet& OrthoContextMenu::getDependencies() const
 {
-    static StringSet _dependencies;
-
-    if (_dependencies.empty())
+    static StringSet _dependencies
     {
-        _dependencies.insert(MODULE_UIMANAGER);
-        _dependencies.insert(MODULE_COMMANDSYSTEM);
-        _dependencies.insert(MODULE_EVENTMANAGER);
-        _dependencies.insert(MODULE_MAINFRAME);
-    }
+        MODULE_COMMANDSYSTEM,
+        MODULE_EVENTMANAGER,
+        MODULE_MAINFRAME,
+    };
 
     return _dependencies;
 }
