@@ -2,7 +2,6 @@
 
 #include "i18n.h"
 #include "imainframe.h"
-#include "iuimanager.h"
 #include "iparticles.h"
 #include "iparticlestage.h"
 #include "igame.h"
@@ -176,10 +175,17 @@ void ParticleEditor::populateParticleDefList()
     // Create and use a ParticlesVisitor to populate the list
     GlobalParticlesManager().forEachParticleDef([&] (const IParticleDef& particle)
 	{
+        auto name = particle.getName();
+
+        if (string::ends_with(name, EDIT_SUFFIX))
+        {
+            return; // skip the edit particles, they should not be showing up
+        }
+
 		// Add the Def name to the list store
         wxutil::TreeModel::Row row = _defList->AddItem();
 
-        row[DEF_COLS().name] = particle.getName();
+        row[DEF_COLS().name] = name;
 
 		row.SendItemAdded();
 	});
@@ -226,36 +232,17 @@ void ParticleEditor::setupParticleStageList()
 		wxEVT_BUTTON, wxCommandEventHandler(ParticleEditor::_onDuplicateStage), NULL, this);
 }
 
-wxSpinCtrlDouble* ParticleEditor::convertToSpinCtrlDouble(wxSpinCtrl* spinCtrlToReplace, double min, double max, double increment, int digits)
-{
-	wxSpinCtrlDouble* spinCtrlDouble = new wxSpinCtrlDouble(spinCtrlToReplace->GetParent(), wxID_ANY);
-	
-	spinCtrlDouble->SetRange(min, max);
-	spinCtrlDouble->SetDigits(digits);
-	spinCtrlDouble->SetIncrement(increment);
-	spinCtrlDouble->SetMaxSize(wxSize(70, -1));
-	
-	wxString name = spinCtrlToReplace->GetName();
-	spinCtrlToReplace->GetContainingSizer()->Replace(spinCtrlToReplace, spinCtrlDouble);
-	spinCtrlToReplace->Destroy();
-
-	spinCtrlDouble->SetName(name);
-	spinCtrlDouble->GetContainingSizer()->Layout();
-
-	return spinCtrlDouble;
-}
-
 wxSpinCtrlDouble* ParticleEditor::convertToSpinCtrlDouble(const std::string& name, double min, double max, double increment, int digits)
 {
-	return convertToSpinCtrlDouble(findNamedObject<wxSpinCtrl>(this, name), min, max, increment, digits);
+	auto spinCtrlDouble = XmlResourceBasedWidget::convertToSpinCtrlDouble(this, name, min, max, increment, digits);
+    spinCtrlDouble->SetMaxSize(wxSize(70, -1));
+    return spinCtrlDouble;
 }
 
 void ParticleEditor::setupSettingsPages()
 {
     // Depth Hack
-	wxSpinCtrlDouble* depthHack = convertToSpinCtrlDouble(
-		findNamedObject<wxSpinCtrl>(this, "ParticleEditorDepthHack"),
-		0, 999, 0.1, 2);
+	wxSpinCtrlDouble* depthHack = convertToSpinCtrlDouble("ParticleEditorDepthHack", 0, 999, 0.1, 2);
 	
 	depthHack->Connect(wxEVT_SPINCTRLDOUBLE, 
 		wxSpinDoubleEventHandler(ParticleEditor::_onDepthHackChanged), NULL, this);
