@@ -3,7 +3,6 @@
 #include "i18n.h"
 #include "iselection.h"
 #include "ieventmanager.h"
-#include "iuimanager.h"
 #include "imap.h"
 #include "iradiant.h"
 #include "imainframe.h"
@@ -14,7 +13,7 @@
 #include <wx/toolbar.h>
 #include <wx/sizer.h>
 #include <wx/stattext.h>
-#include <wx/artprov.h>
+#include "wxutil/Bitmap.h"
 
 namespace ui
 {
@@ -56,7 +55,7 @@ SelectionSetToolmenu::SelectionSetToolmenu() :
 	_dropdownToolId = dropdownTool->GetId();
 
 	_clearAllButton = toolbar->AddTool(CLEAR_TOOL_ID, "",
-		wxArtProvider::GetBitmap(GlobalUIManager().ArtIdPrefix() + "delete.png"), _("Clear Selection Sets"));
+		wxutil::GetLocalBitmap("delete.png"), _("Clear Selection Sets"));
 
 	toolbar->Bind(wxEVT_TOOL, &SelectionSetToolmenu::onDeleteAllSetsClicked, this, _clearAllButton->GetId());
 
