@@ -3,7 +3,7 @@
 
 #include "i18n.h"
 #include "ieventmanager.h"
-#include "iuimanager.h"
+#include "istatusbarmanager.h"
 #include "ipreferencesystem.h"
 #include "itextstream.h"
 
@@ -47,12 +47,14 @@ namespace
 	const std::string RKEY_TRANSLATE_CONSTRAINED = "user/ui/xyview/translateConstrained";
 	const std::string RKEY_FONT_SIZE = "user/ui/xyview/fontSize";
 	const std::string RKEY_FONT_STYLE = "user/ui/xyview/fontStyle";
+	const std::string RKEY_MAX_ZOOM_FACTOR = "user/ui/xyview/maxZoomFactor";
 
     const int DEFAULT_CHASE_MOUSE_CAP = 32; // pixels per chase moue timer interval
 }
 
 // Constructor
-XYWndManager::XYWndManager()
+XYWndManager::XYWndManager() :
+    _maxZoomFactor(1024)
 {}
 
 /* greebo: This method restores all xy views from the information stored in the registry.
@@ -200,6 +202,7 @@ void XYWndManager::constructPreferences()
 	page.appendCheckBox(_("Show Workzone"), RKEY_SHOW_WORKZONE);
 	page.appendCheckBox(_("Translate Manipulator always constrained to Axis"), RKEY_TRANSLATE_CONSTRAINED);
 	page.appendCheckBox(_("Higher Selection Priority for Entities"), RKEY_HIGHER_ENTITY_PRIORITY);
+    page.appendSpinner(_("Maximum Zoom Factor"), RKEY_MAX_ZOOM_FACTOR, 1, 65536, 0);
     page.appendCombo(_("Font Style"), RKEY_FONT_STYLE, { "Sans", "Mono" }, true);
     page.appendSpinner(_("Font Size"), RKEY_FONT_SIZE, 4, 48, 0);
 }
@@ -221,6 +224,7 @@ void XYWndManager::refreshFromRegistry()
 	_defaultBlockSize = registry::getValue<int>(RKEY_DEFAULT_BLOCKSIZE);
     _fontSize = registry::getValue<int>(RKEY_FONT_SIZE);
     _fontStyle = registry::getValue<std::string>(RKEY_FONT_STYLE) == "Sans" ? IGLFont::Style::Sans : IGLFont::Style::Mono;
+    _maxZoomFactor = registry::getValue<float>(RKEY_MAX_ZOOM_FACTOR);
 
 	updateAllViews();
 
@@ -289,6 +293,11 @@ IGLFont::Style XYWndManager::fontStyle() const
     return _fontStyle;
 }
 
+float XYWndManager::maxZoomFactor() const
+{
+    return _maxZoomFactor;
+}
+
 void XYWndManager::updateAllViews(bool force)
 {
 	for (const XYWndMap::value_type& i : _xyWnds)
@@ -647,18 +656,16 @@ const std::string& XYWndManager::getName() const
 
 const StringSet& XYWndManager::getDependencies() const
 {
-	static StringSet _dependencies;
-
-	if (_dependencies.empty())
-	{
-		_dependencies.insert(MODULE_XMLREGISTRY);
-		_dependencies.insert(MODULE_EVENTMANAGER);
-		_dependencies.insert(MODULE_RENDERSYSTEM);
-		_dependencies.insert(MODULE_PREFERENCESYSTEM);
-		_dependencies.insert(MODULE_COMMANDSYSTEM);
-		_dependencies.insert(MODULE_UIMANAGER);
-        _dependencies.insert(MODULE_MOUSETOOLMANAGER);
-	}
+    static StringSet _dependencies
+    {
+        MODULE_XMLREGISTRY,
+        MODULE_EVENTMANAGER,
+        MODULE_RENDERSYSTEM,
+        MODULE_PREFERENCESYSTEM,
+        MODULE_COMMANDSYSTEM,
+        MODULE_MOUSETOOLMANAGER,
+        MODULE_STATUSBARMANAGER
+    };
 
 	return _dependencies;
 }
@@ -691,6 +698,7 @@ void XYWndManager::initialiseModule(const IApplicationContext& ctx)
 	observeKey(RKEY_DEFAULT_BLOCKSIZE);
 	observeKey(RKEY_FONT_SIZE);
 	observeKey(RKEY_FONT_STYLE);
+	observeKey(RKEY_MAX_ZOOM_FACTOR);
 
 	// Trigger loading the values of the observed registry keys
 	refreshFromRegistry();
@@ -701,10 +709,10 @@ void XYWndManager::initialiseModule(const IApplicationContext& ctx)
 	// Add the commands to the EventManager
 	registerCommands();
 
-	GlobalUIManager().getStatusBarManager().addTextElement(
+	GlobalStatusBarManager().addTextElement(
 		"XYZPos",
 		"",  // no icon
-		IStatusBarManager::POS_POSITION,
+		statusbar::StandardPosition::OrthoViewPosition,
 		_("Shows the mouse position in the orthoview")
 	);
 
