@@ -7,6 +7,7 @@
 #include "iclipper.h"
 #include "ientity.h"
 #include "math/Frustum.h"
+#include "math/Hash.h"
 #include <functional>
 
 // Constructor
@@ -20,6 +21,9 @@ BrushNode::BrushNode() :
     _untransformedOriginChanged(true)
 {
 	m_brush.attach(*this); // BrushObserver
+
+    // Try to anticipate a few face additions to avoid reallocations during map parsing
+    reserve(6);
 }
 
 // Copy Constructor
@@ -60,6 +64,44 @@ const AABB& BrushNode::localAABB() const {
 	return m_brush.localAABB();
 }
 
+std::string BrushNode::getFingerprint()
+{
+    constexpr std::size_t SignificantDigits = scene::SignificantFingerprintDoubleDigits;
+
+    if (m_brush.getNumFaces() == 0)
+    {
+        return std::string(); // empty brushes produce an empty fingerprint
+    }
+
+    math::Hash hash;
+    
+    hash.addSizet(static_cast<std::size_t>(m_brush.getDetailFlag() + 1));
+
+    hash.addSizet(m_brush.getNumFaces());
+
+    // Combine all face plane equations
+    for (const auto& face : m_brush)
+    {
+        // Plane equation
+        hash.addVector3(face->getPlane3().normal(), SignificantDigits);
+        hash.addDouble(face->getPlane3().dist(), SignificantDigits);
+
+        // Material Name
+        hash.addString(face->getShader());
+
+        // Texture Matrix
+        auto texdef = face->getTexDefMatrix();
+        hash.addDouble(texdef.xx(), SignificantDigits);
+        hash.addDouble(texdef.yx(), SignificantDigits);
+        hash.addDouble(texdef.tx(), SignificantDigits);
+        hash.addDouble(texdef.xy(), SignificantDigits);
+        hash.addDouble(texdef.yy(), SignificantDigits);
+        hash.addDouble(texdef.ty(), SignificantDigits);
+    }
+
+    return hash;
+}
+
 // Snappable implementation
 void BrushNode::snapto(float snap) {
 	m_brush.snapto(snap);
@@ -73,8 +115,8 @@ void BrushNode::snapComponents(float snap) {
 
 void BrushNode::testSelect(Selector& selector, SelectionTest& test)
 {
-    // BeginMesh(true): Attempt to treat brush faces twosided when in orthoview
-	test.BeginMesh(localToWorld(), true);
+    // BeginMesh(true): Always treat brush faces twosided when in orthoview
+	test.BeginMesh(localToWorld(), !test.getVolume().fill());
 
 	SelectionIntersection best;
 	for (FaceInstances::iterator i = m_faceInstances.begin(); i != m_faceInstances.end(); ++i)
@@ -258,8 +300,9 @@ void BrushNode::reserve(std::size_t size) {
 	m_faceInstances.reserve(size);
 }
 
-void BrushNode::push_back(Face& face) {
-	m_faceInstances.push_back(FaceInstance(face, std::bind(&BrushNode::selectedChangedComponent, this, std::placeholders::_1)));
+void BrushNode::push_back(Face& face)
+{
+	m_faceInstances.emplace_back(face, std::bind(&BrushNode::selectedChangedComponent, this, std::placeholders::_1));
     _untransformedOriginChanged = true;
 }
 
@@ -398,7 +441,8 @@ void BrushNode::evaluateViewDependent(const VolumeTest& volume, const Matrix4& l
 	{
 		// Check if face is filtered before adding to visibility matrix
 		// greebo: Removed localToWorld transformation here, brushes don't have a non-identity l2w
-		if (forceVisible || (i->faceIsVisible() && i->intersectVolume(volume)))
+        // Don't cull backfacing planes to make those faces visible in orthoview (#5465)
+		if (forceVisible || i->faceIsVisible())
 		{
 			*j = true;
 
