@@ -72,7 +72,7 @@ public:
 			if (index != brush::c_brush_maxFaces) {
 					const Winding& winding = getFace().getWinding();
 					std::size_t adjacent = winding.next(index);
-					functor(winding[index].vertex.mid(winding[adjacent].vertex));
+					functor(math::midPoint(winding[index].vertex, winding[adjacent].vertex));
 				}
 		}
 	}
@@ -159,34 +159,3 @@ public:
 
 typedef std::vector<FaceInstance> FaceInstances;
 
-/*class FaceInstanceSet
-{
-	typedef SelectionList<FaceInstance> FaceInstances;
-	FaceInstances m_faceInstances;
-public:
-	void insert(FaceInstance& faceInstance) {
-		m_faceInstances.append(faceInstance);
-	}
-	void erase(FaceInstance& faceInstance) {
-		m_faceInstances.erase(faceInstance);
-	}
-
-	template<typename Functor>
-	void foreach(Functor& functor) {
-		for (FaceInstances::iterator i = m_faceInstances.begin(); i != m_faceInstances.end(); ++i) {
-			functor(*(*i));
-		}
-	}
-
-	bool empty() const {
-		return m_faceInstances.empty();
-	}
-
-	FaceInstance& last() const {
-		return m_faceInstances.back();
-	}
-
-	std::size_t size() const {
-		return m_faceInstances.size();
-	}
-};*/
