@@ -19,7 +19,7 @@ std::string LightShader::m_defaultShader = "";
 // ----- Light Class Implementation -------------------------------------------------
 
 // Constructor
-Light::Light(Doom3Entity& entity,
+Light::Light(SpawnArgs& entity,
              LightNode& owner,
              const Callback& transformChanged,
              const Callback& boundsChanged,
@@ -46,7 +46,7 @@ Light::Light(Doom3Entity& entity,
 // Copy Constructor
 Light::Light(const Light& other,
              LightNode& owner,
-             Doom3Entity& entity,
+             SpawnArgs& entity,
              const Callback& transformChanged,
              const Callback& boundsChanged,
              const Callback& lightRadiusChanged)
@@ -161,7 +161,7 @@ void Light::updateOrigin() {
 
     // Update the transformation matrix
     _owner.localToParent() = Matrix4::getIdentity();
-    _owner.localToParent().translateBy(worldOrigin());
+    _owner.localToParent().translateBy(_originTransformed);
     _owner.localToParent().multiplyBy(m_rotation.getMatrix4());
 
     // Notify all child nodes
@@ -292,7 +292,7 @@ void Light::rotationChanged()
 
     // Update the transformation matrix
     _owner.localToParent() = Matrix4::getIdentity();
-    _owner.localToParent().translateBy(worldOrigin());
+    _owner.localToParent().translateBy(_originTransformed);
     _owner.localToParent().multiplyBy(m_rotation.getMatrix4());
 
     // Notify owner about this
@@ -449,7 +449,7 @@ Doom3LightRadius& Light::getDoom3Radius() {
 
 void Light::renderProjectionPoints(RenderableCollector& collector,
                                    const VolumeTest& volume,
-                                   const Matrix4& localToWorld) const 
+                                   const Matrix4& localToWorld) const
 {
     // Add the renderable light target
     collector.setHighlightFlag(RenderableCollector::Highlight::Primitives, false);
@@ -473,7 +473,7 @@ void Light::renderProjectionPoints(RenderableCollector& collector,
 // Adds the light centre renderable to the given collector
 void Light::renderLightCentre(RenderableCollector& collector,
                               const VolumeTest& volume,
-                              const Matrix4& localToWorld) const 
+                              const Matrix4& localToWorld) const
 {
 	collector.addRenderable(*_rCentre.getShader(), _rCentre, localToWorld);
 }
@@ -557,11 +557,14 @@ Matrix4 Light::getLightTextureTransformation() const
     // into texture coordinates that span the range [0..1] within the light volume.
 
     // Example:
-    // For non-rotated point lights the world point [origin - light_radius] will be 
+    // For non-rotated point lights the world point [origin - light_radius] will be
     // transformed to [0,0,0], whereas [origin + light_radius] will be [1,1,1]
 
     if (isProjected())
     {
+        // Ensure _localToTexture matrix is up to date
+        updateProjection();
+
         // First step: subtract the light origin from the world point
         Matrix4 worldTolight = Matrix4::getTranslation(-getLightOrigin());
 
@@ -589,9 +592,9 @@ Matrix4 Light::getLightTextureTransformation() const
                     1.0f / lightBounds.extents.y(),
                     1.0f / lightBounds.extents.z())
         ));
-        // To get texture coordinates in the range of [0..1], we need to scale down 
+        // To get texture coordinates in the range of [0..1], we need to scale down
         // one more time. [-1..1] is 2 units wide, so scale down by factor 2.
-        // By this time, points within the light volume have been mapped 
+        // By this time, points within the light volume have been mapped
         // into a [-0.5..0.5] cube around the origin.
         worldTolight.premultiplyBy(Matrix4::getScale(Vector3(0.5f, 0.5f, 0.5f)));
 
@@ -616,7 +619,11 @@ AABB Light::lightAABB() const
     }
     else
     {
-        return AABB(_originTransformed, m_doom3Radius.m_radiusTransformed);
+        // AABB ignores light_center so we can't call getLightOrigin() here.
+        // Just transform (0, 0, 0) by localToWorld to get the world origin for
+        // the AABB.
+        return AABB(_owner.localToWorld().transformPoint(Vector3(0, 0, 0)),
+                    m_doom3Radius.m_radiusTransformed);
     }
 }
 
@@ -629,15 +636,21 @@ const Matrix4& Light::rotation() const {
  * the centerTransformed variable as the lighting should be updated as soon as the light center
  * is dragged.
  */
-Vector3 Light::getLightOrigin() const {
+Vector3 Light::getLightOrigin() const
+{
     if (isProjected())
     {
-        return worldOrigin();
+        return _originTransformed;
     }
     else
     {
-        // AABB origin + light_center, i.e. center in world space
-        return worldOrigin() + m_doom3Radius.m_centerTransformed;
+        // Since localToWorld() takes into account our own origin as well as the
+        // transformation of any parent entity, just transform a null origin +
+        // light_center by the localToWorld matrix to get the light origin in
+        // world space.
+        return _owner.localToWorld().transformPoint(
+            /* (0, 0, 0) + */ m_doom3Radius.m_centerTransformed
+        );
     }
 }
 
@@ -700,13 +713,13 @@ void Light::updateProjection() const
 
     Plane3 lightProject[4];
 
-    float rLen = _lightRightTransformed.getLength();
+    auto rLen = _lightRightTransformed.getLength();
     Vector3 right = _lightRightTransformed / rLen;
-    float uLen = _lightUpTransformed.getLength();
+    auto uLen = _lightUpTransformed.getLength();
     Vector3 up = _lightUpTransformed / uLen;
-    Vector3 normal = up.crossProduct(right).getNormalised();
+    Vector3 normal = up.cross(right).getNormalised();
 
-    double dist = _lightTargetTransformed.dot(normal);
+    auto dist = _lightTargetTransformed.dot(normal);
     if ( dist < 0 ) {
         dist = -dist;
         normal = -normal;
@@ -755,7 +768,7 @@ void Light::updateProjection() const
     // Calculate the falloff vector
     Vector3 falloff = stop - start;
 
-    float length = falloff.getLength();
+    auto length = falloff.getLength();
     falloff /= length;
     if ( length <= 0 ) {
         length = 1;
@@ -769,8 +782,8 @@ void Light::updateProjection() const
 	//{
 	//	rMessage() << "  Plane " << i << ": " << lightProject[i].normal() << ", dist: " << lightProject[i].dist() << std::endl;
 	//}
-	
-	// greebo: Comparing this to the engine sources, all frustum planes in TDM 
+
+	// greebo: Comparing this to the engine sources, all frustum planes in TDM
 	// appear to be negated, their normals are pointing outwards.
 
     // we want the planes of s=0, s=q, t=0, and t=q
@@ -796,7 +809,7 @@ void Light::updateProjection() const
     // Normalise all frustum planes
     _frustum.normalisePlanes();
 
-	// TDM uses an array of 6 idPlanes, these relate to DarkRadiant like this: 
+	// TDM uses an array of 6 idPlanes, these relate to DarkRadiant like this:
 	// 0 = left, 1 = top, 2 = right, 3 = bottom, 4 = front, 5 = back
 	//rMessage() << "  Frustum Plane " << 0 << ": " << _frustum.left.normal() << ", dist: " << _frustum.left.dist() << std::endl;
 	//rMessage() << "  Frustum Plane " << 1 << ": " << _frustum.top.normal() << ", dist: " << _frustum.top.dist() << std::endl;
@@ -873,10 +886,4 @@ const IRenderEntity& Light::getLightEntity() const
 	return _owner;
 }
 
-const Vector3& Light::worldOrigin() const
-{
-    // return the absolute world origin
-    return _originTransformed;
-}
-
 } // namespace entity
