@@ -1,6 +1,7 @@
 #pragma once
 
 #include "ilightnode.h"
+#include "registry/CachedKey.h"
 
 #include "Light.h"
 #include "dragplanes.h"
@@ -45,6 +46,9 @@ class LightNode :
 	// a temporary variable for calculating the AABB of all (selected) components
 	mutable AABB m_aabb_component;
 
+    // Cached rkey to override light volume colour
+    registry::CachedKey<bool> _overrideColKey;
+
 public:
 	LightNode(const IEntityClassPtr& eclass);
 
@@ -54,6 +58,9 @@ private:
 public:
 	static LightNodePtr Create(const IEntityClassPtr& eclass);
 
+    // ILightNode implementation
+    const RendererLight& getRendererLight() const { return _light; }
+
 	// RenderEntity implementation
 	virtual float getShaderParm(int parmNum) const override;
 
@@ -114,7 +121,7 @@ public:
 	void renderComponents(RenderableCollector& collector, const VolumeTest& volume) const override;
 
     // OpenGLRenderable implementation
-    void render(const RenderInfo& info) const;
+    void render(const RenderInfo& info) const override;
 
 	const Matrix4& rotation() const;
 
