@@ -13,10 +13,13 @@
 #include "model/export/ScaledModelExporter.h"
 #include "model/export/ModelScalePreserver.h"
 #include "MapPositionManager.h"
+#include "PointFile.h"
 #include "messages/ApplicationShutdownRequest.h"
 
 #include <sigc++/signal.h>
 #include "time/StopWatch.h"
+#include "scene/merge/MergeOperation.h"
+#include "scene/merge/MergeActionNode.h"
 
 class TextInputStream;
 
@@ -31,6 +34,9 @@ class Map :
 	public IMap,
 	public scene::Graph::Observer
 {
+private:
+    EditMode _editMode;
+
 	// The map name
 	std::string _mapName;
 
@@ -39,6 +45,7 @@ class Map :
 
 	sigc::signal<void> _mapNameChangedSignal;
 	sigc::signal<void> _mapModifiedChangedSignal;
+	sigc::signal<void, EditMode> _mapEditModeChangedSignal;
 
 	// Pointer to the resource for this map
 	IMapResourcePtr _resource;
@@ -59,9 +66,15 @@ class Map :
     util::StopWatch _mapSaveTimer;
 
 	MapEventSignal _mapEvent;
-
 	std::size_t _shutdownListener;
 
+    scene::merge::IMergeOperation::Ptr _mergeOperation;
+    std::list<scene::MergeActionNodeBase::Ptr> _mergeActionNodes;
+    sigc::connection _mergeOperationListener;
+
+    // Point trace for leak detection
+    std::unique_ptr<PointFile> _pointTrace;
+
 private:
     std::string getSaveConfirmationText() const;
 
@@ -69,6 +82,12 @@ public:
 	Map();
 
 	MapEventSignal signal_mapEvent() const override;
+
+    EditMode getEditMode() override;
+    void setEditMode(EditMode mode) override;
+
+    sigc::signal<void, EditMode>& signal_editModeChanged() override;
+
 	const scene::INodePtr& getWorldspawn() override;
 	const scene::INodePtr& findOrInsertWorldspawn() override;
 	scene::IMapRootNodePtr getRoot() override;
@@ -82,20 +101,14 @@ public:
 	// Gets called when a node is removed from the scenegraph
 	void onSceneNodeErase(const scene::INodePtr& node) override;
 
-	/** greebo: Returns true if the map has not been named yet.
-	 */
+    // IMap implementation
 	bool isUnnamed() const override;
-
-	/** greebo: Updates the name of the map (and triggers an update
-	 * 			of the mainframe window title)
-	 */
 	void setMapName(const std::string& newName);
-
-	/** greebo: Returns the name of this map
-	 */
 	std::string getMapName() const override;
-
 	sigc::signal<void>& signal_mapNameChanged() override;
+    void forEachPointfile(PointfileFunctor func) const override;
+    void showPointFile(const fs::path& filePath) override;
+    bool isPointTraceVisible() const override;
 
 	/**
 	 * greebo: Saves the current map, doesn't ask for any filenames,
@@ -126,7 +139,7 @@ public:
 	void saveCopyAs();
 
     /**
-     * Saves a copy of the current map to the given path, using the 
+     * Saves a copy of the current map to the given path, using the
      * given format (which may be an empty reference, in which case the map format
      * will be guessed from the filename).
      */
@@ -145,7 +158,7 @@ public:
 	 */
 	bool import(const std::string& filename);
 
-	/** 
+	/**
 	 * greebo: Exports the current map directly to the given filename.
 	 * This skips any "modified" or "unnamed" checks, it just dumps
 	 * the current scenegraph content to the file.
@@ -157,6 +170,12 @@ public:
 	void exportSelected(std::ostream& out) override;
 	void exportSelected(std::ostream& out, const MapFormatPtr& format) override;
 
+    void startMergeOperation(const std::string& sourceMap) override;
+    void startMergeOperation(const std::string& sourceMap, const std::string& baseMap) override;
+    void finishMergeOperation() override;
+    void abortMergeOperation() override;
+    scene::merge::IMergeOperation::Ptr getActiveMergeOperation() override;
+
 	// free all map elements, reinitialize the structures that depend on them
 	void freeMap();
 
@@ -182,10 +201,6 @@ public:
 	 */
 	MapFormatPtr getFormat();
 
-	/** greebo: Focus the XYViews and the Camera to the given point/angle.
-	 */
-	static void focusViews(const Vector3& point, const Vector3& angles);
-
 	/** greebo: Registers the commands with the EventManager.
 	 */
 	void registerCommands();
@@ -210,7 +225,7 @@ public:
 	static void saveSelectedAsPrefab(const cmd::ArgumentList& args);
 
 private:
-	/** 
+	/**
 	 * greebo: Asks the user if the current changes should be saved.
 	 *
 	 * @returns: true, if the user gave clearance (map was saved, had no
@@ -225,7 +240,7 @@ private:
 	void loadPrefabAt(const cmd::ArgumentList& args);
 
 	/**
-	 * greebo: Tries to locate the worldspawn in the global scenegraph and 
+	 * greebo: Tries to locate the worldspawn in the global scenegraph and
 	 * stores it into the local member variable.
 	 * Returns the node that was found (can be an empty ptr).
 	 */
@@ -246,11 +261,25 @@ private:
 	void loadMapResourceFromPath(const std::string& path);
 	void loadMapResourceFromArchive(const std::string& archive, const std::string& archiveRelativePath);
 
+    void startMergeOperationCmd(const cmd::ArgumentList& args);
+    void abortMergeOperationCmd(const cmd::ArgumentList& args);
+    void finishMergeOperationCmd(const cmd::ArgumentList& args);
+
+    void createMergeActions();
+    void prepareMergeOperation();
+    void onMergeActionAdded(const scene::merge::IMergeAction::Ptr& action);
+
 	void emitMapEvent(MapEvent ev);
 
 	void clearMapResource();
 
-}; // class Map
+    void cleanupMergeOperation();
+
+    /** greebo: Focus the XYViews and the Camera to the given point/angle.
+     */
+    void focusViews(const Vector3& point, const Vector3& angles);
+    void focusViewCmd(const cmd::ArgumentList& args);
+};
 
 } // namespace map
 
