@@ -27,7 +27,7 @@ public:
     scene::INodePtr parse(parser::DefTokeniser& tok) const;
 
 private:
-    static Matrix4 getTexDef(float shiftS, float shiftT, float rotation, float scaleS, float scaleT);
+    static Matrix4 getTexDef(std::string shader, float shiftS, float shiftT, float rotation, float scaleS, float scaleT);
 };
 typedef std::shared_ptr<LegacyBrushDefParser> LegacyBrushDefParserPtr;
 
