@@ -146,7 +146,7 @@ void Patch::setDims(std::size_t w, std::size_t h)
   else if(h < MIN_PATCH_HEIGHT)
     h = MIN_PATCH_HEIGHT;
 
-  _width = w; 
+  _width = w;
   _height = h;
 
   if(_width * _height != _ctrl.size())
@@ -185,7 +185,7 @@ void Patch::onAllocate(std::size_t size)
 }
 
 // Return the interally stored AABB
-const AABB& Patch::localAABB() const 
+const AABB& Patch::localAABB() const
 {
     return _localAABB;
 }
@@ -195,7 +195,7 @@ void Patch::renderWireframe(RenderableCollector& collector, const VolumeTest& vo
     // Defer the tesselation calculation to the last minute
     const_cast<Patch&>(*this).updateTesselation();
 
-    collector.addRenderable(*entity.getWireShader(), 
+    collector.addRenderable(*entity.getWireShader(),
         _patchDef3 ? _fixedWireframeRenderable : _wireframeRenderable, localToWorld);
 }
 
@@ -314,7 +314,7 @@ void Patch::freezeTransform()
     // Save the transformed working set array over _ctrl
     _ctrl = _ctrlTransformed;
 
-    // Don't call controlPointsChanged() here since that one will re-apply the 
+    // Don't call controlPointsChanged() here since that one will re-apply the
     // current transformation matrix, possible the second time.
     transformChanged();
     updateTesselation();
@@ -359,9 +359,9 @@ const std::string& Patch::getShader() const
 void Patch::setShader(const std::string& name)
 {
     undoSave();
-    
+
     _shader.setMaterialName(name);
-    
+
     // Check if the shader is ok
     check_shader();
     // Call the callback functions
@@ -386,7 +386,7 @@ bool Patch::hasVisibleMaterial() const
     return material && material->isVisible();
 }
 
-int Patch::getShaderFlags() const 
+int Patch::getShaderFlags() const
 {
     if (_shader.getGLShader() != 0)
     {
@@ -506,7 +506,7 @@ bool Patch::isDegenerate() const {
     for (PatchControlConstIter i = _ctrl.begin(); i != _ctrl.end(); ++i) {
 
         // Skip the first comparison
-        if (i != _ctrl.begin() && !i->vertex.isEqual(prev, 0.0001)) {
+        if (i != _ctrl.begin() && !math::isNear(i->vertex, prev, 0.0001)) {
             return false;
         }
 
@@ -527,7 +527,7 @@ void Patch::updateTesselation()
 
     _ctrl_vertices.clear();
     _latticeIndices.clear();
-    
+
     if (!isValid())
     {
         _mesh.clear();
@@ -542,7 +542,7 @@ void Patch::updateTesselation()
 
     // Generate the indices for the coloured control points and the lines in between
     IndexBuffer ctrl_indices;
-    
+
     _latticeIndices.reserve(((_width * (_height - 1)) + (_height * (_width - 1))) << 1);
     ctrl_indices.reserve(_ctrlTransformed.size());
 
@@ -643,7 +643,7 @@ void Patch::Redisperse(EMatrixMajor mt)
     {
       p2 = p1+col_stride;
       p3 = p2+col_stride;
-      p2->vertex = p1->vertex.mid(p3->vertex);
+      p2->vertex = math::midPoint(p1->vertex, p3->vertex);
       p1 = p3;
     }
   }
@@ -1301,7 +1301,7 @@ void Patch::ConstructSeam(patch::CapType eType, Vector3* p, std::size_t width)
     break;
   case patch::CapType::EndCap:
     {
-      Vector3 p5(p[0].mid(p[4]));
+      Vector3 p5(math::midPoint(p[0], p[4]));
 
       setDims(3, 3);
       _ctrl[0].vertex = p[0];
@@ -1850,7 +1850,7 @@ PatchRenderIndices Patch::getRenderIndices() const
 {
 	// Ensure the tesselation is up to date
 	const_cast<Patch&>(*this).updateTesselation();
-	
+
 	PatchRenderIndices info;
 
 	info.indices = _mesh.indices;
@@ -2319,7 +2319,7 @@ inline void calculateColTangentForCtrl(const Patch& sourcePatch, std::size_t row
         colTangent[1] *= -1;
 
         // Cull redundant tangents
-        if (colTangent[1].isParallel(colTangent[0]))
+        if (math::isParallel(colTangent[1], colTangent[0]))
         {
             colTangent[1] = Vector3(0, 0, 0);
         }
@@ -2359,7 +2359,7 @@ inline void calculateRowTangentForCtrl(const Patch& sourcePatch, std::size_t row
         rowTangent[1] *= -1;
 
         // Cull redundant tangents
-        if (rowTangent[1].isParallel(rowTangent[0]))
+        if (math::isParallel(rowTangent[1], rowTangent[0]))
         {
             rowTangent[1] = Vector3(0, 0, 0);
         }
@@ -2374,8 +2374,8 @@ Vector3 calculateNormalForTangents(Vector3 colTangent[2], Vector3 rowTangent[2],
     if (colTangent[1].getLengthSquared() > 0)
     {
         // Two column normals to calculate
-        Vector3 normal1 = rowTangent[0].crossProduct(colTangent[0]);
-        Vector3 normal2 = rowTangent[0].crossProduct(colTangent[1]);
+        Vector3 normal1 = rowTangent[0].cross(colTangent[0]);
+        Vector3 normal2 = rowTangent[0].cross(colTangent[1]);
 
         if (normal1.getLengthSquared() > 0)
         {
@@ -2398,8 +2398,8 @@ Vector3 calculateNormalForTangents(Vector3 colTangent[2], Vector3 rowTangent[2],
         if (rowTangent[1].getLengthSquared() > 0)
         {
             // Two row normals to calculate
-            Vector3 normal1 = rowTangent[0].crossProduct(colTangent[0]);
-            Vector3 normal2 = rowTangent[1].crossProduct(colTangent[0]);
+            Vector3 normal1 = rowTangent[0].cross(colTangent[0]);
+            Vector3 normal2 = rowTangent[1].cross(colTangent[0]);
 
             if (normal1.getLengthSquared() > 0)
             {
@@ -2418,7 +2418,7 @@ Vector3 calculateNormalForTangents(Vector3 colTangent[2], Vector3 rowTangent[2],
         }
         else
         {
-            normal = rowTangent[0].crossProduct(colTangent[0]);
+            normal = rowTangent[0].cross(colTangent[0]);
 
             if (normal.getLengthSquared() > 0)
             {
@@ -2635,10 +2635,10 @@ void Patch::stitchTextureFrom(Patch& sourcePatch) {
                     // Get the other control
                     const PatchControl& other = sourcePatch.ctrlAt(srcRow, srcCol);
 
-                    float dist = (other.vertex - self.vertex).getLength();
+                    auto dist = (other.vertex - self.vertex).getLength();
 
                     // Allow the coords to be a _bit_ distant
-                    if (fabs(dist) < 0.005f) {
+                    if (fabs(dist) < 0.005) {
                         // Assimilate the texture coordinates
                         self.texcoord = other.texcoord;
                     }
@@ -2693,7 +2693,7 @@ void Patch::normaliseTexture() {
     }
 }
 
-const Subdivisions& Patch::getSubdivisions() const 
+const Subdivisions& Patch::getSubdivisions() const
 {
     return _subDivisions;
 }
