@@ -5,6 +5,7 @@
 #include "iradiant.h"
 #include "icounter.h"
 #include "math/Frustum.h"
+#include "math/Hash.h"
 
 // Construct a PatchNode with no arguments
 PatchNode::PatchNode(patch::PatchDefType type) :
@@ -45,6 +46,42 @@ scene::INode::Type PatchNode::getNodeType() const
 	return Type::Patch;
 }
 
+std::string PatchNode::getFingerprint()
+{
+    constexpr std::size_t SignificantDigits = scene::SignificantFingerprintDoubleDigits;
+
+    if (m_patch.getHeight() * m_patch.getWidth() == 0)
+    {
+        return std::string(); // empty patches produce an empty fingerprint
+    }
+
+    math::Hash hash;
+
+    // Width & Height
+    hash.addSizet(m_patch.getHeight());
+    hash.addSizet(m_patch.getWidth());
+
+    // Subdivision Settings
+    if (m_patch.subdivisionsFixed())
+    {
+        hash.addSizet(static_cast<std::size_t>(m_patch.getSubdivisions().x()));
+        hash.addSizet(static_cast<std::size_t>(m_patch.getSubdivisions().y()));
+    }
+
+    // Material Name
+    hash.addString(m_patch.getShader());
+
+    // Combine all control point data
+    for (const auto& ctrl : m_patch.getControlPoints())
+    {
+        hash.addVector3(ctrl.vertex, SignificantDigits);
+        hash.addDouble(ctrl.texcoord.x(), SignificantDigits);
+        hash.addDouble(ctrl.texcoord.y(), SignificantDigits);
+    }
+
+    return hash;
+}
+
 void PatchNode::allocate(std::size_t size) {
 	// Clear the control instance vector and reserve <size> memory
 	m_ctrl_instances.clear();
@@ -110,12 +147,14 @@ void PatchNode::snapComponents(float snap) {
 }
 
 // Test the Patch instance for selection
-void PatchNode::testSelect(Selector& selector, SelectionTest& test) {
-	// Do not select patch if it is filtered
-	if (!isVisible())
-		return;
+void PatchNode::testSelect(Selector& selector, SelectionTest& test)
+{
+    // Check if this patch has a twosided material
+    const auto& shader = m_patch.getSurfaceShader().getGLShader();
+    bool isTwosided = shader && shader->getMaterial()->getCullType() == Material::CULL_NONE;
+
+    test.BeginMesh(localToWorld(), isTwosided);
 
-    test.BeginMesh(localToWorld(), true);
     // Pass the selection test call to the patch
     m_patch.testSelect(selector, test);
 }
@@ -421,7 +460,7 @@ void PatchNode::transformComponents(const Matrix4& matrix) {
 	// this should only be true when the transform is a pure translation.
 	if (m_dragPlanes.isSelected())
 	{
-		m_patch.transform(m_dragPlanes.evaluateTransform(matrix.t().getVector3()));
+		m_patch.transform(m_dragPlanes.evaluateTransform(matrix.tCol().getVector3()));
 	}
 }
 
