@@ -91,10 +91,7 @@ void RadiantSelectionSystem::testSelectScene(SelectablesList& targetList, Select
             EntitySelector entityTester(selector, test);
             GlobalSceneGraph().foreachVisibleNodeInVolume(view, entityTester);
 
-            for (SelectionPool::const_iterator i = selector.begin(); i != selector.end(); ++i)
-            {
-                targetList.push_back(i->second);
-            }
+            std::for_each(selector.begin(), selector.end(), [&](const auto& p) { targetList.push_back(p.second); });
         }
         break;
 
@@ -121,9 +118,7 @@ void RadiantSelectionSystem::testSelectScene(SelectablesList& targetList, Select
             }
 
             // Add the first selection crop to the target vector
-            for (SelectionPool::const_iterator i = selector.begin(); i != selector.end(); ++i) {
-                targetList.push_back(i->second);
-            }
+            std::for_each(selector.begin(), selector.end(), [&](const auto& p) { targetList.push_back(p.second); });
 
             // Add the secondary crop to the vector (if it has any entries)
             for (SelectionPool::const_iterator i = sel2.begin(); i != sel2.end(); ++i) {
@@ -147,22 +142,26 @@ void RadiantSelectionSystem::testSelectScene(SelectablesList& targetList, Select
             GlobalSceneGraph().foreachVisibleNodeInVolume(view, primitiveTester);
 
             // Add the selection crop to the target vector
-            for (SelectionPool::const_iterator i = selector.begin(); i != selector.end(); ++i)
-            {
-                targetList.push_back(i->second);
-            }
+            std::for_each(selector.begin(), selector.end(), [&](const auto& p) { targetList.push_back(p.second); });
         }
         break;
 
         case eComponent:
         {
             ComponentSelector selectionTester(selector, test, componentMode);
-            foreachSelected(selectionTester);
+            SelectionSystem::foreachSelected(selectionTester);
 
-            for (SelectionPool::const_iterator i = selector.begin(); i != selector.end(); ++i)
-            {
-                targetList.push_back(i->second);
-            }
+            std::for_each(selector.begin(), selector.end(), [&](const auto& p) { targetList.push_back(p.second); });
+        }
+        break;
+
+        case eMergeAction:
+        {
+            MergeActionSelector tester(selector, test);
+            GlobalSceneGraph().foreachVisibleNodeInVolume(view, tester);
+
+            // Add the selection crop to the target vector
+            std::for_each(selector.begin(), selector.end(), [&](const auto& p) { targetList.push_back(p.second); });
         }
         break;
     } // switch
@@ -177,7 +176,7 @@ bool RadiantSelectionSystem::nothingSelected() const
            (Mode() == eGroupPart && _countPrimitive == 0);
 }
 
-void RadiantSelectionSystem::pivotChanged()  
+void RadiantSelectionSystem::pivotChanged()
 {
 	_pivot.setNeedsRecalculation(true);
     SceneChangeNotify();
@@ -345,7 +344,7 @@ void RadiantSelectionSystem::onSelectedChanged(const scene::INodePtr& node, cons
     int delta = isSelected ? +1 : -1;
 
     _countPrimitive += delta;
-    
+
     _selectionInfo.totalCount += delta;
 
     if (Node_isPatch(node))
@@ -356,7 +355,7 @@ void RadiantSelectionSystem::onSelectedChanged(const scene::INodePtr& node, cons
 	{
         _selectionInfo.brushCount += delta;
     }
-    else 
+    else
 	{
         _selectionInfo.entityCount += delta;
     }
@@ -381,7 +380,7 @@ void RadiantSelectionSystem::onSelectedChanged(const scene::INodePtr& node, cons
     ASSERT_MESSAGE(_selection.size() == _countPrimitive, "selection-tracking error");
 
     _requestWorkZoneRecalculation = true;
-   
+
 	// When everything is deselected, release the pivot user lock
 	if (_selection.empty())
 	{
@@ -396,7 +395,7 @@ void RadiantSelectionSystem::onComponentSelection(const scene::INodePtr& node, c
     int delta = selectable.isSelected() ? +1 : -1;
 
     _countComponent += delta;
-    
+
     _selectionInfo.totalCount += delta;
     _selectionInfo.componentCount += delta;
 
@@ -477,17 +476,6 @@ void RadiantSelectionSystem::setSelectedAllComponents(bool selected)
 	_activeManipulator->setSelected(selected);
 }
 
-// Traverse the current selection and visit them with the given visitor class
-void RadiantSelectionSystem::foreachSelected(const Visitor& visitor)
-{
-    for (SelectionListType::const_iterator i = _selection.begin();
-         i != _selection.end();
-         /* in-loop increment */)
-    {
-        visitor.visit((i++)->first);
-    }
-}
-
 // Traverse the current selection components and visit them with the given visitor class
 void RadiantSelectionSystem::foreachSelectedComponent(const Visitor& visitor)
 {
@@ -597,30 +585,30 @@ void RadiantSelectionSystem::selectPoint(SelectionTest& test, EModifier modifier
         }
     }
 
+    // The possible candidates are stored in the SelectablesSet
+    SelectablesList candidates;
+
+    if (face)
     {
-        // The possible candidates are stored in the SelectablesSet
-        SelectablesList candidates;
+        SelectionPool selector;
 
-        if (face)
+        ComponentSelector selectionTester(selector, test, eFace);
+        GlobalSceneGraph().foreachVisibleNodeInVolume(test.getVolume(), selectionTester);
+
+        // Load them all into the vector
+        for (SelectionPool::const_iterator i = selector.begin(); i != selector.end(); ++i)
         {
-            SelectionPool selector;
-
-            ComponentSelector selectionTester(selector, test, eFace);
-            GlobalSceneGraph().foreachVisibleNodeInVolume(test.getVolume(), selectionTester);
-
-            // Load them all into the vector
-            for (SelectionPool::const_iterator i = selector.begin(); i != selector.end(); ++i)
-            {
-                candidates.push_back(i->second);
-            }
+            candidates.push_back(i->second);
         }
-        else {
-            testSelectScene(candidates, test, test.getVolume(), Mode(), ComponentMode());
-        }
-
-        // Was the selection test successful (have we found anything to select)?
-        performPointSelection(candidates, modifier);
     }
+    else {
+        testSelectScene(candidates, test, test.getVolume(), Mode(), ComponentMode());
+    }
+
+    // Was the selection test successful (have we found anything to select)?
+    performPointSelection(candidates, modifier);
+
+    onSelectionPerformed();
 }
 
 namespace algorithm
@@ -648,11 +636,11 @@ void RadiantSelectionSystem::performPointSelection(const SelectablesList& candid
 	if (candidates.empty()) return;
 
 	// Yes, now determine how we should interpret the click
-	switch (modifier) 
+	switch (modifier)
 	{
 		// If we are in toggle mode (Shift-Left-Click by default), just toggle the
 		// selection of the "topmost" item
-		case eToggle: 
+		case eToggle:
 		{
 			ISelectable* best = candidates.front();
 			// toggle selection of the object with least depth (=first in the list)
@@ -691,7 +679,7 @@ void RadiantSelectionSystem::performPointSelection(const SelectablesList& candid
 					// check if there is a "next" item in the list, if not: select the first item
 					++i;
 
-					if (i != candidates.end()) 
+					if (i != candidates.end())
 					{
 						algorithm::setSelectionStatus(*i, true);
 						//(*i)->setSelected(true);
@@ -709,7 +697,7 @@ void RadiantSelectionSystem::performPointSelection(const SelectablesList& candid
 		}
 		break;
 
-		default: 
+		default:
 			break;
 	};
 }
@@ -717,7 +705,7 @@ void RadiantSelectionSystem::performPointSelection(const SelectablesList& candid
 void RadiantSelectionSystem::selectArea(SelectionTest& test, SelectionSystem::EModifier modifier, bool face)
 {
     // If we are in replace mode, deselect all the components or previous selections
-    if (modifier == SelectionSystem::eReplace) 
+    if (modifier == SelectionSystem::eReplace)
     {
         if (face)
         {
@@ -729,44 +717,52 @@ void RadiantSelectionSystem::selectArea(SelectionTest& test, SelectionSystem::EM
         }
     }
 
+    // The posssible candidates go here
+    SelectionPool pool;
+
+    SelectablesList candidates;
+
+    if (face)
     {
-        // The posssible candidates go here
-        SelectionPool pool;
+        ComponentSelector selectionTester(pool, test, eFace);
+        GlobalSceneGraph().foreachVisibleNodeInVolume(test.getVolume(), selectionTester);
 
-        SelectablesList candidates;
-
-        if (face)
+        // Load them all into the vector
+        for (SelectionPool::const_iterator i = pool.begin(); i != pool.end(); ++i)
         {
-            ComponentSelector selectionTester(pool, test, eFace);
-            GlobalSceneGraph().foreachVisibleNodeInVolume(test.getVolume(), selectionTester);
-
-            // Load them all into the vector
-            for (SelectionPool::const_iterator i = pool.begin(); i != pool.end(); ++i)
-            {
-                candidates.push_back(i->second);
-            }
-        }
-        else 
-        {
-            testSelectScene(candidates, test, test.getVolume(), Mode(), ComponentMode());
-        }
-
-        // Since toggling a selectable might trigger a group-selection
-        // we need to keep track of the desired state of each selectable 
-        typedef std::map<ISelectable*, bool> SelectablesMap;
-        SelectablesMap selectableStates;
-
-        for (ISelectable* selectable : candidates)
-        {
-            bool desiredState = !(modifier == SelectionSystem::eToggle && selectable->isSelected());
-            selectableStates.insert(SelectablesMap::value_type(selectable, desiredState));
-        }
-
-        for (const auto& state : selectableStates)
-        {
-            algorithm::setSelectionStatus(state.first, state.second);
+            candidates.push_back(i->second);
         }
     }
+    else
+    {
+        testSelectScene(candidates, test, test.getVolume(), Mode(), ComponentMode());
+    }
+
+    // Since toggling a selectable might trigger a group-selection
+    // we need to keep track of the desired state of each selectable
+    typedef std::map<ISelectable*, bool> SelectablesMap;
+    SelectablesMap selectableStates;
+
+    for (ISelectable* selectable : candidates)
+    {
+        bool desiredState = !(modifier == SelectionSystem::eToggle && selectable->isSelected());
+        selectableStates.insert(SelectablesMap::value_type(selectable, desiredState));
+    }
+
+    for (const auto& state : selectableStates)
+    {
+        algorithm::setSelectionStatus(state.first, state.second);
+    }
+
+    onSelectionPerformed();
+}
+
+void RadiantSelectionSystem::onSelectionPerformed()
+{
+    // #5460: To fix the workzone not being recalculated when the selection changes,
+    // invoke getWorkZone() here. Since the core binary doesn't have any idle processing
+    // anymore, we need to recalculate the workzone after the user is done selecting
+    getWorkZone();
 }
 
 void RadiantSelectionSystem::onManipulationStart()
@@ -803,7 +799,7 @@ void RadiantSelectionSystem::onManipulationEnd()
     }
 
     // Remove all degenerated brushes from the scene graph (should emit a warning)
-    foreachSelected(RemoveDegenerateBrushWalker());
+    SelectionSystem::foreachSelected(RemoveDegenerateBrushWalker());
 
     pivotChanged();
     activeManipulator->setSelected(false);
@@ -853,15 +849,16 @@ void RadiantSelectionSystem::onManipulationCancelled()
         GlobalSceneGraph().root()->traverse(faceSelector);
     }
 
+    _pivot.cancelOperation();
+
     pivotChanged();
 }
 
-void RadiantSelectionSystem::renderWireframe(RenderableCollector& collector, const VolumeTest& volume) const 
+void RadiantSelectionSystem::renderWireframe(RenderableCollector& collector, const VolumeTest& volume) const
 {
     renderSolid(collector, volume);
 }
 
-// Lets the recalculatePivot2World() method do the work and returns the result that is stored in the member variable
 const Matrix4& RadiantSelectionSystem::getPivot2World()
 {
     return _pivot.getMatrix4();
@@ -977,7 +974,7 @@ const StringSet& RadiantSelectionSystem::getDependencies() const
     return _dependencies;
 }
 
-void RadiantSelectionSystem::initialiseModule(const IApplicationContext& ctx) 
+void RadiantSelectionSystem::initialiseModule(const IApplicationContext& ctx)
 {
     rMessage() << getName() << "::initialiseModule called." << std::endl;
 
@@ -1011,10 +1008,11 @@ void RadiantSelectionSystem::initialiseModule(const IApplicationContext& ctx)
 
     GlobalCommandSystem().addCommand("ToggleManipulatorMode",
         std::bind(&RadiantSelectionSystem::toggleManipulatorModeCmd, this, std::placeholders::_1), { cmd::ARGTYPE_STRING });
-    
+
     GlobalCommandSystem().addCommand("ToggleEntitySelectionMode", std::bind(&RadiantSelectionSystem::toggleEntityMode, this, std::placeholders::_1));
     GlobalCommandSystem().addCommand("ToggleGroupPartSelectionMode", std::bind(&RadiantSelectionSystem::toggleGroupPartMode, this, std::placeholders::_1));
-    
+    GlobalCommandSystem().addCommand("ToggleMergeActionSelectionMode", std::bind(&RadiantSelectionSystem::toggleMergeActionMode, this, std::placeholders::_1));
+
     GlobalCommandSystem().addCommand("ToggleComponentSelectionMode",
         std::bind(&RadiantSelectionSystem::toggleComponentModeCmd, this, std::placeholders::_1), { cmd::ARGTYPE_STRING });
 
@@ -1028,7 +1026,7 @@ void RadiantSelectionSystem::initialiseModule(const IApplicationContext& ctx)
 
 	IPreferencePage& page = GlobalPreferenceSystem().getPage(_("Settings/Selection"));
 
-	page.appendCheckBox(_("Ignore light volume bounds when calculating default rotation pivot location"), 
+	page.appendCheckBox(_("Ignore light volume bounds when calculating default rotation pivot location"),
 		ManipulationPivot::RKEY_DEFAULT_PIVOT_LOCATION_IGNORES_LIGHT_VOLUMES);
 
     // Connect the bounds changed caller
@@ -1042,14 +1040,31 @@ void RadiantSelectionSystem::initialiseModule(const IApplicationContext& ctx)
 		sigc::mem_fun(*this, &RadiantSelectionSystem::onMapEvent));
 }
 
-void RadiantSelectionSystem::shutdownModule() 
+void RadiantSelectionSystem::shutdownModule()
 {
     // greebo: Unselect everything so that no references to scene::Nodes
     // are kept after shutdown, causing destruction issues.
     setSelectedAll(false);
     setSelectedAllComponents(false);
 
-	// In pathological cases this list might contain remnants, clear it
+	// In pathological cases this list might contain remnants. First, give each
+	// selectable node a chance to remove itself from the container by setting
+	// its own selected state to false (rather than waiting for this to happen
+	// in its destructor).
+    for (auto i = _selection.begin(); i != _selection.end(); )
+    {
+        // Take a reference to the node and increment the iterator while the
+        // iterator is still valid.
+        scene::INodePtr node = (i++)->first;
+
+        // If this is a selectable node, unselect it (which might remove it from
+        // the map and invalidate the original iterator)
+        auto selectable = Node_getSelectable(node);
+        if (selectable)
+            selectable->setSelected(false);
+    }
+
+    // Clear the list of anything which remains.
 	_selection.clear();
 
 	_activeManipulator.reset();
@@ -1250,7 +1265,7 @@ void RadiantSelectionSystem::toggleEntityMode(const cmd::ArgumentList& args)
 	{
 		activateDefaultMode();
 	}
-	else 
+	else
 	{
 		SetMode(eEntity);
 		SetComponentMode(eDefault);
@@ -1286,7 +1301,7 @@ void RadiantSelectionSystem::toggleGroupPartMode(const cmd::ArgumentList& args)
 
         // Now deselect everything and select all child primitives instead
         setSelectedAll(false);
-        
+
         std::for_each(groupEntityNodes.begin(), groupEntityNodes.end(), [&](const scene::INodePtr& node)
         {
             node->foreachNode([&] (const scene::INodePtr& child)->bool
@@ -1304,6 +1319,32 @@ void RadiantSelectionSystem::toggleGroupPartMode(const cmd::ArgumentList& args)
 	onComponentModeChanged();
 }
 
+void RadiantSelectionSystem::toggleMergeActionMode(const cmd::ArgumentList& args)
+{
+    auto oldMode = Mode();
+
+    if (Mode() == eMergeAction)
+    {
+        activateDefaultMode();
+    }
+    // Only allow switching if the map has an active merge operation
+    else if (GlobalMapModule().getEditMode() == IMap::EditMode::Merge)
+    {
+        // De-select everything when switching to merge mode
+        setSelectedAll(false);
+        setSelectedAllComponents(false);
+
+        SetMode(eMergeAction);
+        SetComponentMode(eDefault);
+    }
+
+    if (oldMode != Mode())
+    {
+        onManipulatorModeChanged();
+        onComponentModeChanged();
+    }
+}
+
 void RadiantSelectionSystem::onManipulatorModeChanged()
 {
     _sigActiveManipulatorChanged.emit(getActiveManipulatorType());
