@@ -1,10 +1,12 @@
 #include "Doom3ShaderSystem.h"
 #include "ShaderFileLoader.h"
+#include "MaterialSourceGenerator.h"
 
 #include "i18n.h"
 #include "iradiant.h"
 #include "iregistry.h"
 #include "ifilesystem.h"
+#include "ifiletypes.h"
 #include "ipreferencesystem.h"
 #include "imainframe.h"
 #include "ieventmanager.h"
@@ -21,21 +23,19 @@
 #include "debugging/ScopedDebugTimer.h"
 #include "module/StaticModule.h"
 
+#include "os/file.h"
+#include "os/path.h"
+#include "decl/SpliceHelper.h"
+#include "stream/TemporaryOutputStream.h"
 #include "string/predicate.h"
 #include "string/replace.h"
+#include "materials/ParseLib.h"
 #include "parser/DefBlockTokeniser.h"
 #include <functional>
 
-namespace 
+namespace
 {
     const char* TEXTURE_PREFIX = "textures/";
-    const char* MISSING_BASEPATH_NODE =
-        "Failed to find \"/game/filesystem/shaders/basepath\" node \
-in game descriptor";
-
-    const char* MISSING_EXTENSION_NODE =
-        "Failed to find \"/game/filesystem/shaders/extension\" node \
-in game descriptor";
 
     // Default image maps for optional material stages
     const std::string IMAGE_FLAT = "_flat.bmp";
@@ -73,7 +73,7 @@ void Doom3ShaderSystem::destroy()
     GlobalFileSystem().removeObserver(*this);
 
     // Free the shaders if we're in realised state
-    if (_realised) 
+    if (_realised)
     {
         freeShaders();
     }
@@ -85,30 +85,17 @@ void Doom3ShaderSystem::destroy()
 ShaderLibraryPtr Doom3ShaderSystem::loadMaterialFiles()
 {
     // Get the shaders path and extension from the XML game file
-    xml::NodeList nlShaderPath =
-        GlobalGameManager().currentGame()->getLocalXPath("/filesystem/shaders/basepath");
-    if (nlShaderPath.empty())
-        throw xml::MissingXMLNodeException(MISSING_BASEPATH_NODE);
-
-    xml::NodeList nlShaderExt =
-        GlobalGameManager().currentGame()->getLocalXPath("/filesystem/shaders/extension");
-    if (nlShaderExt.empty())
-        throw xml::MissingXMLNodeException(MISSING_EXTENSION_NODE);
+    auto materialsFolder = getMaterialsFolderName();
+    auto extension = getMaterialFileExtension();
 
     // Load the shader files from the VFS
-    std::string sPath = nlShaderPath[0].getContent();
-    if (!string::ends_with(sPath, "/"))
-        sPath += "/";
-
-    std::string extension = nlShaderExt[0].getContent();
-
     ShaderLibraryPtr library = std::make_shared<ShaderLibrary>();
 
     // Load each file from the global filesystem
     {
         ScopedDebugTimer timer("ShaderFiles parsed: ");
         ShaderFileLoader<ShaderLibrary> loader(GlobalFileSystem(), *library,
-                                               sPath, extension);
+            materialsFolder, extension);
         loader.parseFiles();
     }
 
@@ -119,7 +106,7 @@ ShaderLibraryPtr Doom3ShaderSystem::loadMaterialFiles()
 
 void Doom3ShaderSystem::realise()
 {
-    if (!_realised) 
+    if (!_realised)
     {
         // Start loading defs
         _defLoader.start();
@@ -188,7 +175,7 @@ sigc::signal<void>& Doom3ShaderSystem::signal_DefsUnloaded()
 }
 
 // Return a shader by name
-MaterialPtr Doom3ShaderSystem::getMaterialForName(const std::string& name)
+MaterialPtr Doom3ShaderSystem::getMaterial(const std::string& name)
 {
     ensureDefsLoaded();
 
@@ -203,6 +190,19 @@ bool Doom3ShaderSystem::materialExists(const std::string& name)
     return _library->definitionExists(name);
 }
 
+bool Doom3ShaderSystem::materialCanBeModified(const std::string& name)
+{
+    ensureDefsLoaded();
+
+    if (!_library->definitionExists(name))
+    {
+        return false;
+    }
+
+    const auto& def = _library->getDefinition(name);
+    return def.file.name.empty() || def.file.getIsPhysicalFile();
+}
+
 void Doom3ShaderSystem::foreachShaderName(const ShaderNameCallback& callback)
 {
     ensureDefsLoaded();
@@ -245,19 +245,19 @@ GLTextureManager& Doom3ShaderSystem::getTextureManager()
 }
 
 // Get default textures
-TexturePtr Doom3ShaderSystem::getDefaultInteractionTexture(ShaderLayer::Type type)
+TexturePtr Doom3ShaderSystem::getDefaultInteractionTexture(IShaderLayer::Type type)
 {
     TexturePtr defaultTex;
 
     // Look up based on layer type
     switch (type)
     {
-    case ShaderLayer::DIFFUSE:
-    case ShaderLayer::SPECULAR:
+    case IShaderLayer::DIFFUSE:
+    case IShaderLayer::SPECULAR:
         defaultTex = _textureManager->getBinding(getBitmapsPath() + IMAGE_BLACK);
         break;
 
-    case ShaderLayer::BUMP:
+    case IShaderLayer::BUMP:
         defaultTex = _textureManager->getBinding(getBitmapsPath() + IMAGE_FLAT);
         break;
     default:
@@ -296,12 +296,230 @@ TexturePtr Doom3ShaderSystem::loadTextureFromFile(const std::string& filename)
     return _textureManager->getBinding(filename);
 }
 
-IShaderExpressionPtr Doom3ShaderSystem::createShaderExpressionFromString(const std::string& exprStr)
+sigc::signal<void, const std::string&>& Doom3ShaderSystem::signal_materialCreated()
+{
+    return _sigMaterialCreated;
+}
+
+sigc::signal<void, const std::string&, const std::string&>& Doom3ShaderSystem::signal_materialRenamed()
+{
+    return _sigMaterialRenamed;
+}
+
+sigc::signal<void, const std::string&>& Doom3ShaderSystem::signal_materialRemoved()
+{
+    return _sigMaterialRemoved;
+}
+
+IShaderExpression::Ptr Doom3ShaderSystem::createShaderExpressionFromString(const std::string& exprStr)
 {
     return ShaderExpression::createFromString(exprStr);
 }
 
-TableDefinitionPtr Doom3ShaderSystem::getTableForName(const std::string& name)
+std::string Doom3ShaderSystem::ensureNonConflictingName(const std::string& name)
+{
+    auto candidate = name;
+    auto i = 0;
+
+    while (_library->definitionExists(candidate))
+    {
+        candidate += fmt::format("{0:02d}", ++i);
+    }
+
+    return candidate;
+}
+
+MaterialPtr Doom3ShaderSystem::createEmptyMaterial(const std::string& name)
+{
+    auto candidate = ensureNonConflictingName(name);
+
+    // Create a new template/definition
+    auto shaderTemplate = std::make_shared<ShaderTemplate>(candidate, "");
+
+    ShaderDefinition def{ shaderTemplate, vfs::FileInfo("", "", vfs::Visibility::HIDDEN) };
+
+    _library->addDefinition(candidate, def);
+
+    auto material = _library->findShader(candidate);
+    material->setIsModified();
+
+    _sigMaterialCreated.emit(candidate);
+
+    return material;
+}
+
+bool Doom3ShaderSystem::renameMaterial(const std::string& oldName, const std::string& newName)
+{
+    if (oldName == newName)
+    {
+        rWarning() << "Cannot rename, the new name is no different" << std::endl;
+        return false;
+    }
+
+    if (!_library->definitionExists(oldName))
+    {
+        rWarning() << "Cannot rename non-existent material " << oldName << std::endl;
+        return false;
+    }
+
+    if (_library->definitionExists(newName))
+    {
+        rWarning() << "Cannot rename material to " << newName << " since this name is already in use" << std::endl;
+        return false;
+    }
+
+    _library->renameDefinition(oldName, newName);
+
+    _sigMaterialRenamed(oldName, newName);
+
+    return true;
+}
+
+void Doom3ShaderSystem::removeMaterial(const std::string& name)
+{
+    if (!_library->definitionExists(name))
+    {
+        rWarning() << "Cannot remove non-existent material " << name << std::endl;
+        return;
+    }
+
+    _library->removeDefinition(name);
+
+    _sigMaterialRemoved.emit(name);
+}
+
+MaterialPtr Doom3ShaderSystem::createDefaultMaterial(const std::string& name)
+{
+    return std::make_shared<CShader>(name, _library->getEmptyDefinition(), true);
+}
+
+MaterialPtr Doom3ShaderSystem::copyMaterial(const std::string& nameOfOriginal, const std::string& nameOfCopy)
+{
+    if (nameOfCopy.empty())
+    {
+        rWarning() << "Cannot copy, the new name must not be empty" << std::endl;
+        return MaterialPtr();
+    }
+
+    auto candidate = ensureNonConflictingName(nameOfCopy);
+
+    if (!_library->definitionExists(nameOfOriginal))
+    {
+        rWarning() << "Cannot copy non-existent material " << nameOfOriginal << std::endl;
+        return MaterialPtr();
+    }
+
+    _library->copyDefinition(nameOfOriginal, candidate);
+
+    _sigMaterialCreated.emit(candidate);
+
+    auto material = _library->findShader(candidate);
+    material->setIsModified();
+
+    return material;
+}
+
+void Doom3ShaderSystem::saveMaterial(const std::string& name)
+{
+    ensureDefsLoaded();
+
+    auto material = _library->findShader(name);
+
+    if (!material->isModified())
+    {
+        rMessage() << "Material is not modified, nothing to save." << std::endl;
+        return;
+    }
+
+    if (!materialCanBeModified(material->getName()))
+    {
+        throw std::runtime_error("Cannot save this material, it's read-only.");
+    }
+
+    if (material->getShaderFileInfo().fullPath().empty())
+    {
+        throw std::runtime_error("No file path set on this material, cannot save.");
+    }
+
+    // Construct the output path for this material
+    fs::path outputPath = GlobalGameManager().getModPath();
+
+    if (outputPath.empty())
+    {
+        outputPath = GlobalGameManager().getUserEnginePath();
+    }
+
+    outputPath /= material->getShaderFileInfo().fullPath();
+
+    auto outputDir = os::getDirectory(outputPath.string());
+    if (!os::fileOrDirExists(outputDir))
+    {
+        rMessage() << "Creating mod materials path: " << outputDir << std::endl;
+        fs::create_directories(outputDir);
+    }
+
+    rMessage() << "Saving material " << material->getName() << " to " << outputPath << std::endl;
+
+    stream::TemporaryOutputStream tempStream(outputPath);
+
+    std::string tempString;
+    auto& stream = tempStream.getStream();
+
+    // If a previous file exists, open it for reading and filter out the material def we'll be writing
+    if (fs::exists(outputPath))
+    {
+        std::ifstream inheritStream(outputPath);
+
+        if (!inheritStream.is_open())
+        {
+            throw std::runtime_error(
+                fmt::format(_("Cannot open file for reading: {0}"), outputPath.string()));
+        }
+
+        // Write the file to the output stream, up to the point the material def should be written to
+        std::regex pattern(getDeclNamePatternForMaterialName(material->getName()));
+
+        decl::SpliceHelper::PipeStreamUntilInsertionPoint(inheritStream, stream, pattern);
+
+        if (inheritStream.eof())
+        {
+            // Material declaration was not found in the inherited stream, write our comment
+            stream << std::endl << std::endl;
+
+            MaterialSourceGenerator::WriteMaterialGenerationComment(stream);
+        }
+
+        // We're at the insertion point (which might as well be EOF of the inheritStream)
+
+        // Write the name, curly braces and block contents
+        MaterialSourceGenerator::WriteFullMaterialToStream(stream, material);
+
+        stream << inheritStream.rdbuf();
+
+        inheritStream.close();
+    }
+    else
+    {
+        // File is empty, just write the comment and the declaration
+        MaterialSourceGenerator::WriteMaterialGenerationComment(stream);
+        MaterialSourceGenerator::WriteFullMaterialToStream(stream, material);
+    }
+
+    tempStream.closeAndReplaceTargetFile();
+
+    // Store the modifications in our actual template and un-mark the file
+    material->commitModifications();
+
+    // Update the template in our library
+    // Re-acquire the vfs::FileInfo structure which might still be empty for a newly created material
+    _library->replaceDefinition(material->getName(), ShaderDefinition
+    { 
+        material->getTemplate(), 
+        GlobalFileSystem().getFileInfo(material->getShaderFileInfo().fullPath())
+    });
+}
+
+ITableDefinition::Ptr Doom3ShaderSystem::getTable(const std::string& name)
 {
     ensureDefsLoaded();
 
@@ -323,6 +541,7 @@ const StringSet& Doom3ShaderSystem::getDependencies() const
         _dependencies.insert(MODULE_VIRTUALFILESYSTEM);
         _dependencies.insert(MODULE_XMLREGISTRY);
         _dependencies.insert(MODULE_GAMEMANAGER);
+        _dependencies.insert(MODULE_FILETYPES);
     }
 
     return _dependencies;
@@ -338,6 +557,9 @@ void Doom3ShaderSystem::initialiseModule(const IApplicationContext& ctx)
 #if 0
     testShaderExpressionParsing();
 #endif
+
+    // Register the mtr file extension
+    GlobalFiletypes().registerPattern("material", FileTypePattern(_("Material File"), "mtr", "*.mtr"));
 }
 
 // Horrible evil macro to avoid assertion failures if expr is NULL
@@ -348,7 +570,7 @@ void Doom3ShaderSystem::testShaderExpressionParsing()
 {
     // Test a few things
     std::string exprStr = "3";
-    IShaderExpressionPtr expr;
+    IShaderExpression::Ptr expr;
     GET_EXPR_OR_RETURN;
     rMessage() << "Expression " << exprStr << ": " << expr->getValue(0) << std::endl;
 
